#include "global.h"
#include "TimerOne.h"
#include "Sensor.h"
#include "Motor.h"



const String VERSION = "0.1;";


bool bReadSensor;

// Kommunikation -----
String inputString;
String ret;
long watchdog = 0;


// Sensoren -----
Sensor sensorAZ('A');
Sensor sensorEL('E');

// Motoren -----
Motor motorAZ('A');
Motor motorEL('E');
Motor motorLK('L'); //Lockmotor


// ###################################################################
void setup() {
  pinMode(10, INPUT);
  
  // initialize serial and wait for port to open:
  Serial.begin(9600);
  while (!Serial) {
    ; // wait for serial port to connect. Needed for native USB port only
  }


  Timer1.initialize(20);                  // 50.000 Aufrufe / sec
  Timer1.attachInterrupt(timerCallback);  // attaches callback() as a timer overflow interrupt

  inputString = ""; //für neues Kommando bereitmachen



/*
motorAZ.parseCommand("MATS");
motorEL.parseCommand("METS");
sensorAZ.parseCommand("SATI");
sensorEL.parseCommand("SETI");
*/
}


// ----------------------------------------------------------------
void loop() {
  if (Serial.available()) {
    char inChar = (char)Serial.read();
    inputString += inChar; //Einzelzeichen des Kommandos zusammensammeln.

    if (inChar == ';') {
      switch (inputString.charAt(0)) {
        case 'V':   //Versionsnummer
          Serial.print(VERSION);
          break;

        case 'S':   //Sensor
          sensorAZ.parseCommand(inputString);
          sensorEL.parseCommand(inputString);
          Serial.print(";");  //Empfangsbestätigung
          break;

        case 'M':   //Motor
          motorAZ.parseCommand(inputString);
          sensorAZ.setSensorMotorparameter(motorAZ.motorDrehrichtung, motorAZ.motorOn);
          //sensorAZ.setSensorImpulsZaehlrichtung(motorAZ.motorDrehrichtung);
          
          motorEL.parseCommand(inputString);
          sensorEL.setSensorMotorparameter(motorEL.motorDrehrichtung, motorEL.motorOn);
          //sensorEL.setSensorImpulsZaehlrichtung(motorEL.motorDrehrichtung);
          
          Serial.print(";");  //Empfangsbestätigung
          break;

        case 'G':   //Get
          ret = getData(inputString.charAt(1));
          Serial.print(ret + ";");  //Werterückgabe
          break;

        default:
          Serial.print(inputString);
      }
      inputString = ""; //für neues Kommando bereitmachen
      watchdog = 0;
    }
  }

  if (bReadSensor) {
    sensorAZ.readSensor();
    sensorEL.readSensor();
    bReadSensor = false;
  }

  //Überwachung der Kommunkkation; Werden für ca. 10 sec keine Kommandos übertrage, werden alle Ausgänge abgeschalten.
  if (watchdog > 500000) {
    motorAZ.looseConnection();
    motorEL.looseConnection();
    sensorAZ.looseConnection();
    sensorEL.looseConnection();
  }
}


// -------------------------------------------------------------------
String getData(char datenArt) {
  String r;
  
  switch (datenArt) {
    case 'A':
      r = String(sensorAZ.sensorValue);
      r += sensorAZ.sensorIsSynchrone ? "s" : "n";
      return r;
      break;
    case 'E':
      r = String(sensorEL.sensorValue);
      r += sensorEL.sensorIsSynchrone ? "s" : "n";
      return r;
      break;

    case 'd':   // debug
      r = String(10) + 0xff;
      r += sensorAZ.sensorIsSynchrone ? "s" : "n";
      return r;
      break;
  }
}



// Timer ######################################################################
void timerCallback() {
  // Schrittmotor - Takt -------------------------
  motorAZ.stepMotor();
  motorEL.stepMotor();

  bReadSensor = true;
  watchdog++;
}
