/*
 * Motoren
 */

#ifndef MOTOR_cpp
#define MOTOR_cpp

#include "global.h"
#include "Motor.h"


// private #############################################################

void Motor::setMotorTyp() {
  switch (motorTyp) {
    case 'A': // Analogmotor
      pinMode(pinMotorPower    , OUTPUT);
      pinMode(pinMotorDirection, OUTPUT);
      pinMode(pinMotorSpeed    , OUTPUT);
      digitalWrite(pinMotorPower    , LOW);
      digitalWrite(pinMotorDirection, LOW);
      digitalWrite(pinMotorSpeed    , LOW);
      break;
    case 'S': // Schrittmotor
      pinMode(pinMotorPower    , OUTPUT);
      pinMode(pinMotorDirection, OUTPUT);
      pinMode(pinMotorSpeed    , INPUT_PULLUP);
      digitalWrite(pinMotorPower    , LOW);
      digitalWrite(pinMotorDirection, LOW);
      break;
    default:
      pinMode(pinMotorPower    , INPUT);
      pinMode(pinMotorDirection, INPUT);
      pinMode(pinMotorSpeed    , INPUT);
  }
}



// public #############################################################

Motor::Motor(char achse) {
  motorAchse = achse;
  
  switch (motorAchse) {
    case 'A': // AZ
      pinMotorPower = 5;
      pinMotorDirection = 6;
      pinMotorSpeed = 7;
      break;
    case 'E': // EL
      pinMotorPower = 2;
      pinMotorDirection = 3;
      pinMotorSpeed = 4;
      break;  
    case 'L': // Lock
      pinMotorPower = 8;
      pinMotorDirection = 9;
      pinMotorSpeed = 13;
      break;
  }

  setMotorTyp();
}


void Motor::parseCommand(String cmd) {
  if (cmd.charAt(1) != motorAchse) return;  //Prüfen ob das Kommando für diese Achse ist.

  bool b;

  switch (cmd.charAt(2)) {
    case 'T': //Typ setzen
      motorTyp = cmd.charAt(3);
      setMotorTyp();
      if (cmd.charAt(4) == 'i') {    //Invertieren des Ausgangspins für die Motordrehrichtung
        motorDrehrichtungPinInvertieren = true;
      }
      break;

    case 'C': //Geschwindigkeit des Schrittmotors setzen
      stepperClock = str2long(cmd, 3);
      motorOn = (stepperClock < STEPPERMOTOROFF);
      break;

    case 'D': //Drehrichtung setzen
      motorDrehrichtung = (cmd.charAt(3) == '1');
      if (motorDrehrichtungPinInvertieren) {
        digitalWrite(pinMotorDirection, !motorDrehrichtung);  // Drehrichtunspin Invertieren
      }
      else {
        digitalWrite(pinMotorDirection, motorDrehrichtung);
      }
      break;

    case 'S': //Geschwindigkeit des Analogmotors setzen
      stepperClock = STEPPERMOTOROFF;
      b = (cmd.charAt(3) == '1');
      digitalWrite(pinMotorSpeed, b);
      break;

    case 'P': //Aus-Ein Schalten
      stepperClock = STEPPERMOTOROFF;
      motorOn = (cmd.charAt(3) == '1');
      digitalWrite(pinMotorPower, motorOn);
      break;
  }
}



void Motor::stepMotor() {
  if (stepperDelay == 0) {
    if (motorTyp == 'S' && stepperClock < STEPPERMOTOROFF) {
      digitalWrite(pinMotorPower, digitalRead(pinMotorPower) ^ 1);
      stepperDelay = stepperClock;
    }
  }
  else {
    stepperDelay--;
  }
}


void Motor::looseConnection() {
  digitalWrite(pinMotorPower    , LOW);
  digitalWrite(pinMotorDirection, LOW);
  digitalWrite(pinMotorSpeed    , LOW);
  motorTyp = '?';
  setMotorTyp();
}

#endif
