/*
 * Positionssensoren
 */

#ifndef SENSOR_cpp
#define SENSOR_cpp

#include "global.h"
#include "Sensor.h"


// private #############################################################

void Sensor::setSensorTyp() {
  switch (sensorTyp) {
    case 'I': // Impulssensor
      pinMode(pinSensorEnable, INPUT_PULLUP);
      pinMode(pinSensorSync, INPUT_PULLUP);
      pinMode(pinSensorImpuls, INPUT_PULLUP);
      break;
    case 'S': // SPI-Schnittstelle
      pinMode(pinSensorEnable, OUTPUT);
      pinMode(pinSensorClock, OUTPUT);
      pinMode(pinSensorData, INPUT_PULLUP);
      break;
    case 's': // Simulator
      pinMode(pinSensorEnable, INPUT_PULLUP);
      pinMode(pinSensorClock, INPUT_PULLUP);
      pinMode(pinSensorData, INPUT_PULLUP);
      break;
    default:
      pinMode(pinSensorEnable, INPUT_PULLUP);
      pinMode(pinSensorClock, INPUT_PULLUP);
      pinMode(pinSensorData, INPUT_PULLUP);
  }
}


// public #############################################################

Sensor::Sensor(char achse) {
  sensorAchse = achse;
  
  switch (sensorAchse) {
    case 'A': // AZ
      pinSensorEnable = 16;
      pinSensorClock = 15;
      pinSensorData = 14;
      pinSensorSync = 15;
      pinSensorImpuls = 14;
      break;
    case 'E': // EL
      pinSensorEnable = 19;
      pinSensorClock = 18;
      pinSensorData = 17;
      pinSensorSync = 18;
      pinSensorImpuls = 17;
      break;  
  }

  setSensorTyp();
}


void Sensor::parseCommand(String cmd) {
  if (cmd.charAt(1) != sensorAchse) return;

  switch (cmd.charAt(2)) {
    case 'T': //Typ setzen
      sensorTyp = cmd.charAt(3);
      setSensorTyp();
      break;

    case 'S': //Sensorwert setzen
      sensorValue = str2long(cmd, 3);
      break;

    case 'F': //Sensor auf Synchronisiert setzen
      sensorIsSynchrone = true;
      break;

    case 'I': //Sensorwert/Initialwert am Synchpunkt setzen
      sensorSync = str2long(cmd, 3);
      break;

  }
}




void Sensor::setSensorImpulsZaehlrichtung(bool dir) {
  sensorImpulsZaehlrichtung = dir ? 1 : -1;
}


void Sensor::setSensorMotorparameter(bool impulsZaehlrichtung, bool simulatorOn) {
  sensorImpulsZaehlrichtung = impulsZaehlrichtung ? 1 : -1;
  sensorImpulsZaehlrichtung *= simulatorOn ? 1 : 0;
}


void Sensor::readSensor() {
  switch (sensorTyp) {
    case 'I': // Impulssensor
      if (digitalRead(pinSensorSync) == LOW) {
        sensorValue = sensorSync;
        sensorIsSynchrone = true;
      }
      if (digitalRead(pinSensorImpuls) == LOW) {
        if (readBitFromSensor) {
          sensorValue += sensorImpulsZaehlrichtung;
          readBitFromSensor = false;
        }
      }
      else {
        readBitFromSensor = true;
      }
      break;

    case 'S': // SPI-Schnittstelle
      //pinMode(pinSensorEnable, OUTPUT);
      //pinMode(pinSensorClock, OUTPUT);
      //pinMode(pinSensorData, INPUT_PULLUP);
      break;

    case 's': // Simulator
      if (digitalRead(pinSensorSync) == LOW) {
        sensorValue = sensorSync;
        sensorIsSynchrone = true;
      }
      if ((sensorSimulatorDelay > 1000) && (digitalRead(pinSensorImpuls) == HIGH)) {
        sensorValue += sensorImpulsZaehlrichtung;
        sensorSimulatorDelay = 0;
      }
      sensorSimulatorDelay++;
      break;      
  }
}


void Sensor::looseConnection() {
  sensorTyp = '?';
  setSensorTyp();
}

#endif
