
import argparse
import urllib
import urllib.request
import serial           # pip3 install pyserial


def get_parameter():
    # Parameter Parsen
    parser = argparse.ArgumentParser(description="simple script to demonstrate argparse usage")
    parser.add_argument('-s',
                        help="Serielle Schnittatelle. Z.B: COM8 oder /dev/tnt2",
                        type=str,
                        required=True
                        )
    parser.add_argument('-u',
                        help="URL zum RaspberryPi mit Clipeum. Z.B: http://192.168.1.99:4444",
                        type=str,
                        required=True
                        )
    arguments = parser.parse_args()
    print("Serial port:", arguments.s)
    print("Clipeum URL:", arguments.u + "/lx200/")
    return arguments.s, arguments.u + "/lx200/"


def read_serial(ser):
    answer = []
    timeoutcounter = 1000
    while timeoutcounter > 0:
        c = ser.read().decode("ascii")
        if c not in ("", " "):             # Zeichen gelesen
            answer.append(c)    # Zeichen speichern
            if c in ('#', chr(0x06)):        # Abschlusszeichen -> Schleife beenden
                timeoutcounter = 0
        else:
            if timeoutcounter <= 0:  # Nix empfangen -> Timeout
                print("Serial Timeout: " + " ".join(answer))
                raise serial.serialutil.SerialException
            else:                    # kein Zeichen empfangen -> weiter auf Zeichen warten / Timeoutzähler++
                timeoutcounter -= 1

    r = "".join(answer)
    print(r.ljust(18), end='', flush=True)
    return r


def write_serial(ser, message):
    print(message.decode("utf-8"))
    ser.write(message)


if __name__ == '__main__':
    print("")
    print("Start LX200-Serial to Clipeum-URL")
    port, url = get_parameter()
    print("Beenden mit Strg+C")
    print("")

    ser = serial.Serial()
    ser.port = port
    ser.baudrate = 9600
    ser.timeout = 0.5
    ser.bytesize = serial.EIGHTBITS
    ser.parity = serial.PARITY_NONE
    ser.stopbits = serial.STOPBITS_ONE
    ser.open()

    _run = True

    while _run:
        try:
            cmd = read_serial(ser)

            if cmd in (chr(0x06), "#"):
                cmd = '%06'

            print(".", end='', flush=True)
            ans = urllib.request.urlopen(url + cmd).read()
            print(". ", end='', flush=True)
            write_serial(ser, ans)

        except ConnectionRefusedError:
            print("  Keine Verbindung zum Clipeum-RPI .")

        except ConnectionResetError:
            print("  Eine vorhandene Verbindung wurde vom Clipeum-RPI geschlossen.")

        except urllib.error.URLError:
            print("  Keine Verbindung zum Clipeum-RPI.")

        except KeyboardInterrupt:
            _run = False

    ser.close()
    print("")
    print("End LX200-Serial to Clipeum-URL")

# cd C:\Users\B\Documents\LX200serial_to_clipeumURL
# python LX200serial_to_clipeumURL.py -s COM4 -u http://10.0.0.21:4444
