"""
Konfigurationsparameter
"""


from logger import log
import database
import skyfield
from skyfield import api, units


class Configuration(object):
    def __init__(self, db):
        self.__db = db

        self.name = "?"
        self.montierungstyp = "?"
        self.startmodus = "?"
        self.interfaceport = "?"
        self.serverport = 0
        self.nachtpositionschwelle = 0.01
        self.standort_laenge = None
        self.standort_breite = None
        self.standort = None

        self.planets = skyfield.api.load('de421.bsp')

        self.__load_parameter()

    def __del__(self):
        self.close()

    def close(self):
        pass


    def set_standort(self, breitengrad, laengengrad):
        self.standort_laenge = skyfield.units.Angle(degrees=laengengrad)
        self.standort_breite = skyfield.units.Angle(degrees=breitengrad)

        _earth = self.planets['earth']
        brt = str(abs(self.standort_breite.degrees))
        brt += " N" if self.standort_breite.degrees >= 0 else " S"
        lng = str(abs(self.standort_laenge.degrees))
        lng += " E" if self.standort_laenge.degrees >= 0 else " W"
        self.standort = _earth + skyfield.api.Topos(brt, lng)


    def __load_parameter(self):
        self.name = self.__db.get_parameter_str("name")
        self.montierungstyp = self.__db.get_parameter_str("montierungstyp")
        self.startmodus = self.__db.get_parameter_str("startmodus")
        self.interfaceport = self.__db.get_parameter_str("interfaceport")
        self.serverport = self.__db.get_parameter_int("serverport")
        self.nachtpositionschwelle = self.__db.get_parameter_float("nachtpositionschwelle")

        self.timescale = skyfield.api.load.timescale()

        # Standort
        _l = self.__db.get_parameter_float("standortlaenge")
        _b = self.__db.get_parameter_float("standortbreite")
        self.set_standort(_b, _l)
       


if __name__ == '__main__':
    db = database.Database()
    c = Configuration(db)
    print("name:", c.name)
    print("montierungstyp:", c.montierungstyp)
    print("startmodus:", c.startmodus)
    print("interfaceport:", c.interfaceport)
    print("serverpath:", c.serverport)
    print("nachtpositionschwelle:", c.nachtpositionschwelle)
    print("standortlaenge:", c.standort_laenge)
    print("standortbreite:", c.standort_breite)
    db.close()
    c.close()

