"""
Webinterface von Clipeum.
"""


from logger import log
import skyfield
from skyfield import api, units
import flask
import time
from threading import Thread
import io
from PIL import Image, ImageDraw, ImageFont
import numpy as np


class Controlserver(Thread):
    def __init__(self, mount):
        Thread.__init__(self)

        self.__mount = mount
        self.lx200_highpresitiondisplay = True

        self.__app = flask.Flask(__name__)
        self.__route()

        self.daemon = True
        self.start()

    def __del__(self):
        self.close()

    def close(self):
        if self.isAlive():
            self.join(0.2)

    def __route(self):
        self.__app.add_url_rule('/', view_func=self.index)
        self.__app.add_url_rule('/index', view_func=self.index)

        self.__app.add_url_rule('/monitor', view_func=self.monitor)
        self.__app.add_url_rule('/telemetrie', view_func=self.telemetrie)
        self.__app.add_url_rule('/control', view_func=self.control)
        self.__app.add_url_rule('/control_handbox', methods=['GET', 'POST'], view_func=self.control_handbox)
        self.__app.add_url_rule('/config/main', methods=['GET', 'POST'], view_func=self.config_main)
        self.__app.add_url_rule('/config/intervall', methods=['GET', 'POST'], view_func=self.config_intervall)
        self.__app.add_url_rule('/config/fixposition', methods=['GET', 'POST'], view_func=self.config_fixposition)
        self.__app.add_url_rule('/config/motor', methods=['GET', 'POST'], view_func=self.config_motor)
        self.__app.add_url_rule('/config/sensor', methods=['GET', 'POST'], view_func=self.config_sensor)
        self.__app.add_url_rule('/config/transform', methods=['GET', 'POST'], view_func=self.config_transform)
        self.__app.add_url_rule('/config/transformlinearity', methods=['GET', 'POST'],
                                view_func=self.config_transform_linearity)
        self.__app.add_url_rule('/config/transformimage', view_func=self.config_transform_image)
        self.__app.add_url_rule('/lx200test', view_func=self.lx200test)
        self.__app.add_url_rule('/lx200/<cmdh>', view_func=self.lx200)

        #TODO Bereichsscanner umsetzen

    def index(self):
        # log.debug("INDEX")
        cdict = {'title': self.__mount.config.name}
        if str(type(self.__mount)).find("Init_MountDummy") > 0:
            cdict['init_clipeum'] = '1'

        try:
            return flask.render_template('index.html', **cdict)
        except BrokenPipeError as e:
            log.error("BrokenPipeError: " + str(e))
        except IOError as e:
            log.error("IOError: " + e)
        except Exception as e:
            log.error("Exception: " + str(e))


    def monitor(self):
        cdict = {'title': self.__mount.config.name}
        return flask.render_template('monitor.html', **cdict)


    def control(self):
        cdict = {'title': self.__mount.config.name}
        return flask.render_template('control.html', **cdict)


    def control_handbox(self):
        key = flask.request.form.get('key', '')
        pos = flask.request.form.get('pos', '')
        az = flask.request.form.get('az', '180')
        el = flask.request.form.get('el', '45')
        ra = flask.request.form.get('ra', '12')
        de = flask.request.form.get('de', '10')
        glo = flask.request.form.get('glo', '67.5')
        gla = flask.request.form.get('gla', '19.5')

        if key != '':
            log.info("control_handbox: " + key)

            if key == "5":      # Stop
                self.__mount.stop()

            elif key == "6":    # Pfeil-rechts
                self.__mount.move_az_right(True)

            elif key == "4":    # Pfeil-links
                self.__mount.move_az_left(True)

            elif key == "8":    # Pfeil-oben
                self.__mount.move_el_up(True)

            elif key == "2":    # Pfeil-unten
                self.__mount.move_el_down(True)

            elif key == "+":    # Fast
                self.__mount.set_motorrate_manuell(True)

            elif key == "-":    # Slow
                self.__mount.set_motorrate_manuell(False)

            elif key == "1":    # Unlock
                # todo
                pass  # self.__mount.target.set_target(self.__mount.target.TARGET_PARK)

            elif key == "GoToAZEL":   # GoTo AZ-EL
                a = skyfield.units.Angle(degrees=float(az))
                e = skyfield.units.Angle(degrees=float(el))
                self.__mount.target.set_target_altaz(a, e)

            elif key == "GoToRADE":   # GoTo RA-DEC
                r = skyfield.units.Angle(hours=float(ra))
                d = skyfield.units.Angle(degrees=float(de))
                self.__mount.target.set_target_tracking(r, d)

            elif key == "GoToGLATGLON":   # GoTo Galactic Lat-Lon
                r, d = self.__galactic_to_equatorial(float(gla), float(glo))
                r = skyfield.units.Angle(hours=r/15)
                d = skyfield.units.Angle(degrees=d)
                self.__mount.target.set_target_tracking(r, d)

        if pos != '':
            log.info("control_handbox: " + pos)

            if pos == "track":    # Tracking
                ts = skyfield.api.load.timescale()
                rade = self.__mount.config.standort.at(
                    ts.now()).from_altaz(alt_degrees=self.__mount.el.position.degrees,
                                         az_degrees=self.__mount.az.position.degrees)
                _ra, _dec, _distance = rade.radec()
                self.__mount.target.set_target_tracking(_ra, _dec)

            else:
                self.__mount.target.set_target(pos)

        positionsall = self.__mount.db.get_position_all()
        __cnt = (len(positionsall) - 4) // 2
        __cnt = max(__cnt, 0)
        positions1 = positionsall[:__cnt]
        positions2 = positionsall[__cnt:]
        return flask.render_template('control_handbox.html', az=az, el=el, ra=ra, de=de, gla=gla, glo=glo, positions1=positions1, positions2=positions2)


    def __galactic_to_equatorial(self, galactic_latitude, galactic_longitude):
        #Convertercode by https://astrophysicsandpython.com/2021/04/19/celestial-coordinate-conversions/
        _ra_gal, _dec_gal, _long_ncp = np.radians([192.85948, 27.12825, 122.93192])
        
        gal_lat, gal_long = np.radians([galactic_latitude, galactic_longitude])
        dec = np.sin(gal_lat) * np.sin(_dec_gal)
        dec = dec + (np.cos(gal_lat) * np.cos(_dec_gal) * np.cos(_long_ncp - gal_long))
        dec = np.arcsin(dec)
        _ra = np.sin(gal_lat) * np.cos(_dec_gal)
        _ra = _ra - (np.cos(gal_lat) * np.sin(_dec_gal) * np.cos(_long_ncp - gal_long))
        ra = np.arctan2(np.cos(gal_lat) * np.sin(_long_ncp - gal_long), _ra)
        ra = ra + _ra_gal
        ra, dec = np.degrees([ra, dec])
        return ra, dec


    def telemetrie(self):
        if self.__mount.target.target is None:
            target = "automatik"
            targetcolor = "modautomaktik"
        elif self.__mount.target.target == self.__mount.target.TARGET_SUN:
            target = "Sonne"
            targetcolor = "modobject"
        elif self.__mount.target.target == self.__mount.target.TARGET_MOON:
            target = "Mond"
            targetcolor = "modobject"
        elif self.__mount.target.target == self.__mount.target.TARGET_JUPITER:
            target = "Jupiter"
            targetcolor = "modobject"
        elif self.__mount.target.target == self.__mount.target.TARGET_TRACKING:
            target = "tracking"
            targetcolor = "modtracking"

        # elif self.__mount.target.target == self.__mount.target.TARGET_PARK:
        #     target = "Park"
        #     targetcolor = "modlock"

        # elif self.__mount.target.target == self.__mount.target.TARGET_MAINTAIN:
        #     target = "--> " + self.__mount.target.target
        #     targetcolor = "modmaintain"
        #
        # elif self.__mount.target.target == self.__mount.target.TARGET_EMERGENCY:
        #     target = "--> " + self.__mount.target.target
        #     targetcolor = "modmaintain"

        else:
            target = self.__mount.target.target_name
            targetcolor = "modtarget"

        rate_nord = rate_ost = rate_sued = rate_west = "emtypic.gif"
        if self.__mount.el.motor_direction_ist > 0:
            if self.__mount.el.motor_rate_ist == 2:
                rate_nord = "arrowfast.gif"
            elif self.__mount.el.motor_rate_ist == 1:
                rate_nord = "arrowslow.gif"
        elif self.__mount.el.motor_direction_ist < 0:
            if self.__mount.el.motor_rate_ist == 2:
                rate_sued = "arrowfast.gif"
            elif self.__mount.el.motor_rate_ist == 1:
                rate_sued = "arrowslow.gif"

        if self.__mount.az.motor_direction_ist < 0:
            if self.__mount.az.motor_rate_ist == 2:
                rate_ost = "arrowfast.gif"
            elif self.__mount.az.motor_rate_ist == 1:
                rate_ost = "arrowslow.gif"
        elif self.__mount.az.motor_direction_ist > 0:
            if self.__mount.az.motor_rate_ist == 2:
                rate_west = "arrowfast.gif"
            elif self.__mount.az.motor_rate_ist == 1:
                rate_west = "arrowslow.gif"

        cdict = {'pos_ra': self.__mount.terminal.pos_ra,
                 'pos_de': self.__mount.terminal.pos_de,
                 'pos_az': self.__mount.terminal.pos_az,
                 'pos_el': self.__mount.terminal.pos_el,
                 'pos_az_red': "red" if self.__mount.az.grenzverletzung else "",
                 'pos_el_red': "red" if self.__mount.el.grenzverletzung else "",
                 'ziel_ra': self.__mount.terminal.ziel_ra,
                 'ziel_de': self.__mount.terminal.ziel_de,
                 'ziel_az': self.__mount.terminal.ziel_az,
                 'ziel_el': self.__mount.terminal.ziel_el,
                 'nachtpos': "Nachtpos." if self.__mount.target.target_nachtposition else "",
                 'target': target,
                 'targetcolor': targetcolor,
                 'utc': time.strftime("%d.%m.%Y  %H:%M:%S", time.gmtime()),
                 'lng': self.__mount.terminal.laengengrad,
                 'brt': self.__mount.terminal.breitengrad,
                 'sensor_az': self.__mount.az.sensorvalue,
                 'sensor_el': self.__mount.el.sensorvalue,
                 'sensor_az_color': "" if self.__mount.az.sensor_issynchronized else "unsync",
                 'sensor_el_color': "" if self.__mount.el.sensor_issynchronized else "unsync",
                 'rate_nord': rate_nord,
                 'rate_ost': rate_ost,
                 'rate_sued': rate_sued,
                 'rate_west': rate_west
                 }
        return flask.render_template('telemetrie.html', **cdict)


    def config_main(self):
        if flask.request.method == 'POST' and flask.request.form['speichern'] == 'j':       # speichern
            if self.__mount.config.name != flask.request.form['name']:
                self.__mount.config.name = flask.request.form['name']
                self.__mount.db.save_parameter('name', self.__mount.config.name)

            if self.__mount.config.startmodus != flask.request.form['startmodus']:
                self.__mount.config.startmodus = flask.request.form['startmodus']
                self.__mount.db.save_parameter('startmodus', self.__mount.config.startmodus)

            if self.__mount.config.interfaceport != flask.request.form['interfaceport']:
                self.__mount.config.interfaceport = flask.request.form['interfaceport']
                self.__mount.db.save_parameter('interfaceport', self.__mount.config.interfaceport)

            if self.__mount.config.serverport != flask.request.form['serverport']:
                self.__mount.config.serverport = flask.request.form['serverport']
                self.__mount.db.save_parameter('serverport', self.__mount.config.serverport)

            if self.__mount.config.nachtpositionschwelle != flask.request.form['nachtpositionschwelle']:
                self.__mount.config.nachtpositionschwelle = flask.request.form['nachtpositionschwelle']
                self.__mount.db.save_parameter('nachtpositionschwelle', self.__mount.config.nachtpositionschwelle)

            if self.__mount.config.standort_breite.degrees != flask.request.form['standortbreite']:
                self.__mount.config.standort_breite = skyfield.units.Angle(
                    degrees=float(flask.request.form['standortbreite']))
                self.__mount.db.save_parameter('standortbreite', self.__mount.config.standort_breite.degrees)

            if self.__mount.config.standort_laenge.degrees != flask.request.form['standortlaenge']:
                self.__mount.config.standort_laenge = skyfield.units.Angle(
                    degrees=float(flask.request.form['standortlaenge']))
                self.__mount.db.save_parameter('standortlaenge', self.__mount.config.standort_laenge.degrees)

        cdict = {'name': self.__mount.config.name,
                 'startmodus_' + self.__mount.config.startmodus: 'x',
                 'interfaceport': self.__mount.config.interfaceport,
                 'serverport': self.__mount.config.serverport,
                 'nachtpositionschwelle': self.__mount.config.nachtpositionschwelle,
                 'standortbreite': self.__mount.config.standort_breite.degrees,
                 'standortlaenge': self.__mount.config.standort_laenge.degrees
                 }
        return flask.render_template('config_main.html', **cdict)

    def config_intervall(self):
        if flask.request.method == 'POST' and flask.request.form['speichern'] == 'j':       # speichern
            for obj in flask.request.form:
                if obj != "speichern":
                    self.__mount.db.save_intervall(obj, flask.request.form[obj])

        intervalls = self.__mount.db.get_intervall_all()
        return flask.render_template('config_intervall.html', name=self.__mount.config.name, intervalls=intervalls)

    def config_fixposition(self):
        if flask.request.method == 'POST':
            if flask.request.form.get('addpos', '') == 'j':       # Position hinzugügen
                self.__mount.db.add_new_position()

            elif flask.request.form.get('speichern', '') == 'j':  # speichern
                for pos in flask.request.form:
                    if pos[0:1] != "_" and pos != 'speichern':
                        self.__mount.db.save_position(pos,
                                                      flask.request.form['_a_' + pos],
                                                      flask.request.form['_e_' + pos],
                                                      flask.request.form['_b_' + pos])
            else:
                for pos in flask.request.form:         # Position löschen
                    if pos[0:1] != "_":
                        if flask.request.form.get('_d_' + pos, '') == 'j':
                            self.__mount.db.delete_position(pos)

        positions = self.__mount.db.get_position_all()
        return flask.render_template('config_fixposition.html', name=self.__mount.config.name, positions=positions)

    def config_motor(self):
        if flask.request.method == 'POST' and flask.request.form['speichern'] == 'j':       # speichern
            for ax in ('A', 'E'):
                self.__mount.db.save_achsparameter(ax, 'motortyp', flask.request.form[ax + 'motortyp'])
                self.__mount.db.save_achsparameter(ax, 'motorrichtunginvers',
                                                   flask.request.form.get(ax + 'motorrichtunginvers', 'd'))
                self.__mount.db.save_achsparameter(ax, 'schwellerate', flask.request.form[ax + 'schwellerate'])
                self.__mount.db.save_achsparameter(ax, 'schwelleziel', flask.request.form[ax + 'schwelleziel'])
                self.__mount.db.save_achsparameter(ax, 'motormaxfreq', flask.request.form[ax + 'motormaxfreq'])
                self.__mount.db.save_achsparameter(ax, 'grenze-', flask.request.form[ax + 'grenzem'])
                self.__mount.db.save_achsparameter(ax, 'grenze+', flask.request.form[ax + 'grenzep'])

        axis = {}
        for ax in ('A', 'E'):
            _x = {'motortyp_' + self.__mount.db.get_achsparameter_str(ax, "motortyp"): 'x',
                  'motorrichtunginvers': self.__mount.db.get_achsparameter_str(ax, "motorrichtunginvers"),
                  'schwellerate': self.__mount.db.get_achsparameter_float(ax, "schwellerate"),
                  'schwelleziel': self.__mount.db.get_achsparameter_float(ax, "schwelleziel"),
                  'motormaxfreq': self.__mount.db.get_achsparameter_int(ax, "motormaxfreq"),
                  'grenzem': self.__mount.db.get_achsparameter_float(ax, "grenze-"),
                  'grenzep': self.__mount.db.get_achsparameter_float(ax, "grenze+")
                  }
            axis[ax] = _x

        return flask.render_template('config_motor.html', name=self.__mount.config.name, axis=axis)

    def config_sensor(self):
        if flask.request.method == 'POST' and flask.request.form['speichern'] == 'j':       # speichern
            for ax in ('A', 'E'):
                self.__mount.db.save_achsparameter(ax, 'sensortyp', flask.request.form[ax + 'sensortyp'])
                self.__mount.db.save_achsparameter(ax, 'sensorinit', flask.request.form[ax + 'sensorinit'])
                self.__mount.db.save_achsparameter(ax, 'sensorsync', flask.request.form[ax + 'sensorsync'])
                self.__mount.db.save_achsparameter(ax, 'sensorsync-', flask.request.form[ax + 'sensorsyncm'])
                self.__mount.db.save_achsparameter(ax, 'sensorsync+', flask.request.form[ax + 'sensorsyncp'])
                self.__mount.db.save_achsparameter(ax, 'sensorsyncdelay', flask.request.form[ax + 'sensorsyncdelay'])

        axis = {}
        for ax in ('A', 'E'):
            _x = {'sensortyp_' + self.__mount.db.get_achsparameter_str(ax, "sensortyp"): 'x',
                  'sensorinit': self.__mount.db.get_achsparameter_float(ax, "sensorinit"),
                  'sensorsync': self.__mount.db.get_achsparameter_float(ax, "sensorsync"),
                  'sensorsyncm': self.__mount.db.get_achsparameter_float(ax, "sensorsync-"),
                  'sensorsyncp': self.__mount.db.get_achsparameter_float(ax, "sensorsync+"),
                  'sensorsyncdelay': self.__mount.db.get_achsparameter_int(ax, "sensorsyncdelay")
                  }
            axis[ax] = _x

        return flask.render_template('config_sensor.html', name=self.__mount.config.name, axis=axis)


    def config_transform(self):
        if flask.request.method == 'POST':
            #log.debug("config_transform: " + str(flask.request.form))
            
            delbtn = flask.request.form.get('delete', '')       # Referenzpunkt löschen
            savbtn = flask.request.form.get('save', '')         # Referenzpunkt ändern

            if delbtn != '':  # Referenzpunkt löschen
                if delbtn[0:2] == 'az':
                    self.__mount.db.delete_transform_refpoint_az(delbtn[2:])
                elif delbtn[0:2] == 'el':
                    self.__mount.db.delete_transform_refpoint_el(delbtn[2:])

            elif savbtn != '':  # Referenzpunkt ändern
                if savbtn[0:2] == 'az':
                    winkel = float(flask.request.form['azval' + str(savbtn[2:])])
                    self.__mount.db.update_transform_refpoint_az(savbtn[2:], winkel)
                elif savbtn[0:2] == 'el':
                    winkel = float(flask.request.form['elval' + str(savbtn[2:])])
                    self.__mount.db.update_transform_refpoint_el(savbtn[2:], winkel)

            elif flask.request.form.get('addref', '') != '':    # Referenzpunkt hinzufügen
                self.__config_add_transform_refpoint(flask.request.form['addref'], flask.request.form)

            elif flask.request.form.get('transform', '') != '':    # Referenzpunkte in Transformtabelle übernehmen
                self.__mount.stop() # Wärend der Neuberechnung Montierung stoppen
                self.__mount.sensor2angle.calculate_transform_el()
                self.__mount.stop() # Wärend der Neuberechnung Montierung stoppen
                self.__mount.sensor2angle.calculate_transform_az()
                self.__mount.db.reset_transform_newrefpoint()
            
        azchecked = "checked" if 'az' in flask.request.form.getlist('achse') else ""
        elchecked = "checked" if 'el' in flask.request.form.getlist('achse') else ""

        referenzpunkte = self.__mount.db.get_transform_refpoint_all()
        referenzsterne = self.__mount.db.get_referenzstern_all()
        return flask.render_template('config_transform.html',
                                     name=self.__mount.config.name,
                                     issync_az=self.__mount.az.sensor_issynchronized,
                                     issync_el=self.__mount.el.sensor_issynchronized,
                                     az_checked=azchecked,
                                     el_checked=elchecked,
                                     referenzpunkte=referenzpunkte,
                                     RPazGrad=flask.request.form.get('azGrad', '180'),
                                     RPelGrad=flask.request.form.get('elGrad', '45'),
                                     az_delete_allowed=(len(referenzpunkte["az"]) > 2),
                                     el_delete_allowed=(len(referenzpunkte["el"]) > 2),
                                     referenzsterne=referenzsterne)


    def __config_add_transform_refpoint(self, button, request):
        log.debug("Add Referenzpoint: " + button + " " + str(request.getlist('achse')))

        tnow = skyfield.api.load.timescale().now()
        az = skyfield.units.Angle(degrees=-999)
        el = skyfield.units.Angle(degrees=-999)
        point = None

        if button == 'sun':
            point = self.__mount.config.planets['sun']
        elif button == 'moon':
            point = self.__mount.config.planets['moon']
        elif button == 'jupiter':
            point = self.__mount.config.planets['JUPITER BARYCENTER']
        elif button == 'azel':  # AZ+EL
            #az = skyfield.units.Angle(degrees=(int(request['azGrad']), int(request['azMin']), int(request['azSec'])))
            #el = skyfield.units.Angle(degrees=(int(request['elGrad']), int(request['elMin']), int(request['elSec'])))
            az.degrees = float(request['azGrad'])
            el.degrees = float(request['elGrad'])
            #log.debug("Add Referenzpoint1: " + button + " " + request['azGrad'] + " " + request['elGrad'])
            #log.debug("Add Referenzpoint2: " + button + " " + str(az.degrees) + " " + str(el.degrees))
        elif button == 'rade':  # RA+Dec
            point = skyfield.api.Star(ra_hours=(int(request['raHour']), int(request['raMin']), int(request['raSec'])),
                                      dec_degrees=(int(request['deGrad']), int(request['deMin']), int(request['deSec']))
                                      )
        else:
            starpos = self.__mount.db.get_referenzstern_position(button)
            point = skyfield.api.Star(ra_hours=starpos['ra'], dec_degrees=starpos['dec'])

        if point is not None:
            astrometric = self.__mount.config.standort.at(tnow).observe(point)
            el, az, __dis = astrometric.apparent().altaz()

        if 'az' in request.getlist('achse') and self.__mount.az.sensor_issynchronized:
            self.__mount.db.add_new_transform_refpoint_az(self.__mount.az.sensorvalue, az.degrees)
        if 'el' in request.getlist('achse') and self.__mount.el.sensor_issynchronized:
            self.__mount.db.add_new_transform_refpoint_el(self.__mount.el.sensorvalue, el.degrees)


    def config_transform_linearity(self):
        if flask.request.method == 'POST':
            btn = flask.request.form.get('refresh', '')       # neu Zeichnen
            if btn != '':
                self.__mount.db.save_parameter('_transformimage_vscalieren_az_', flask.request.form['vscal_az'])
                self.__mount.db.save_parameter('_transformimage_vscalieren_el_', flask.request.form['vscal_el'])

        return flask.render_template('config_transform_linearity.html',
                                     name=self.__mount.config.name,
                                     vscal_az=self.__mount.db.get_parameter_float('_transformimage_vscalieren_az_'),
                                     vscal_el=self.__mount.db.get_parameter_float('_transformimage_vscalieren_el_'),
                                     refresh_hack=str(time.time()))


    def config_transform_image(self):
        """Grafische Darstellung der Abweichungen der Transformationspunkte von der Linearität."""
        img = Image.new('RGB', (760, 500), 'black')

        # get a drawing context
        idraw = ImageDraw.Draw(img)

        linearity_divergence = self.__mount.db.get_transform_linearity_divergence()
        vscal_az = self.__mount.db.get_parameter_float('_transformimage_vscalieren_az_')
        vscal_el = self.__mount.db.get_parameter_float('_transformimage_vscalieren_el_')

        for l in linearity_divergence["az"]:
            # log.debug("AZ: " + str(l[0]) + " " + str(l[1]) + " " + str(l[2]))
            winkel = (l[0] * 2) + 20
            minabw = (-l[1] / vscal_az) + 125
            maxabw = (-l[2] / vscal_az) + 125
            idraw.line([(winkel, minabw), (winkel, maxabw)], fill='green', width=1)

        for l in linearity_divergence["el"]:
            # log.debug("EL: " + str(l[0]) + " " + str(l[1]) + " " + str(l[2]))
            winkel = (l[0] * 4) + 20
            minabw = (-l[1] / vscal_el) + 375
            maxabw = (-l[2] / vscal_el) + 375
            idraw.line([(winkel, minabw), (winkel, maxabw)], fill='green', width=1)

        # get a font
        fnt1 = ImageFont.truetype('LiberationSans-Regular.ttf', 18)
        fnt2 = ImageFont.truetype('LiberationSans-Regular.ttf', 13)
        fnt3 = ImageFont.truetype('LiberationSans-Regular.ttf', 11)
        # Aufteilung
        idraw.text((6, 10), "AZ", font=fnt1, fill='white')
        idraw.text((6, 260), "EL", font=fnt1, fill='white')
        idraw.text((616, 212), "0.5° pro Pixel Horizontal\n" + str(vscal_az) + "° pro Pixel Vertical",
                   font=fnt3, fill='white')
        idraw.text((610, 462), "0.25° pro Pixel Horizontal\n" + str(vscal_el) + "° pro Pixel Vertical",
                   font=fnt3, fill='white')
        idraw.line([(0, 250), (760, 250)], fill='white', width=3)
        # Grundlinien
        idraw.line([(20, 125), (740, 125)], fill='blue', width=1)
        idraw.line([(20, 375), (540, 375)], fill='blue', width=1)
        idraw.text((2, 125), "0°", font=fnt2, fill='yellow')
        idraw.text((730, 125), "360°", font=fnt2, fill='yellow')
        idraw.text((2, 375), "-20°", font=fnt2, fill='yellow')
        idraw.text((530, 375), "110°", font=fnt2, fill='yellow')
        p = [(20, 125), (200, 125), (380, 125), (560, 125), (740, 125),
             (20, 375), (100, 375), (280, 375), (460, 375), (540, 375)]
        idraw.point(p, fill='yellow')

        img_io = io.BytesIO()
        img.save(img_io, 'JPEG', quality=100)
        img_io.seek(0)
        return flask.send_file(img_io, mimetype='image/jpeg')


    def lx200test(self):
        cmd = flask.request.args.get('cmd', '')
        ans = self.lx200(cmd)

        cdict = {'title': self.__mount.config.name,
                 'lx200_cmd': cmd,
                 'lx200_ans': ans}
        return flask.render_template('lx200test.html', **cdict)


    def lx200(self, cmdh):
        log.debug("URL: " + cmdh)
        cmd = cmdh.rstrip('#')

        # Kommandos parsen
        if cmdh in (chr(0x06), ""):   # ACK ------------------------------------------
                return 'A'

        elif len(cmd) >= 2:
            if cmd[1] == "G":         # Get ------------------------------------------
                return self.__lx200_get(cmd)

            elif cmd[1] == "M":       # Move ------------------------------------------
                return self.__lx200_move(cmd)

            elif cmd[1] == "Q":       # Stop ------------------------------------------
                return self.__lx200_stop(cmd)

            elif cmd[1] == "R":       # Rate ------------------------------------------
                return self.__lx200_rate(cmd)

            elif cmd[1] == "S":       # Set ------------------------------------------
                return self.__lx200_set(cmd)

            elif cmd == ":U":
                self.lx200_highpresitiondisplay = not self.lx200_highpresitiondisplay
                return ""

            elif cmd == "quit":     # Clipeum beenden -------------------------------
                self.__mount.exit_clipeum = True

            elif cmd == "sun":      # der Sonne folgen -------------------------------
                self.__mount.target.set_target(self.__mount.target.TARGET_SUN)

            elif cmd == "moon":     # Mond folgen -------------------------------
                self.__mount.target.set_target(self.__mount.target.TARGET_MOON)

            elif cmd == "jupiter":  # Jupiter folgen -------------------------------
                self.__mount.target.set_target(self.__mount.target.TARGET_JUPITER)

        else:
            return 'CMD-Fehler: ' + cmd

        return ""


    def __lx200_get(self, cmd):
        if cmd == ":GA":       # EL ------------------------------------------
            w = self.__mount.el.position.signed_dms()
            sig = '+' if w[0] >= 0 else '-'
            dd = str(int(w[1])).rjust(2, '0')
            mm = str(int(w[2])).rjust(2, '0')
            ans = sig + dd + "*" + mm
            if self.lx200_highpresitiondisplay:
                ss = str(int(w[3])).rjust(2, '0')
                ans += '’' + ss
            return ans + "#"

        elif cmd == ":GD":     # DE ------------------------------------------
            #ts = skyfield.api.load.timescale()
            #rade = self.__mount.config.standort.at(ts.now()).from_altaz(alt_degrees=self.__mount.el.position.degrees,
            #                                                            az_degrees=self.__mount.az.position.degrees)
            #_ra, _dec, _distance = rade.radec()
            _ra, _dec, _distance = self.__mount.terminal.position.radec()
            w = _dec.signed_dms()
            sig = '+' if w[0] >= 0 else '-'
            dd = str(int(w[1])).rjust(2, '0')
            mm = str(int(w[2])).rjust(2, '0')
            #ans = sig + dd + "*" + mm
            if self.lx200_highpresitiondisplay:
                #ans = sig + dd + ":" + mm
                ss = str(int(w[3])).rjust(2, '0')
                ans = sig + dd + ":" + mm + ":" + ss
            else:
                ans = sig + dd + "*" + mm
            return ans + "#"

        elif cmd == ":Gg":     # Standort Längenkreis. Ost ist Negativ (wegen der Amis) --------------------------------
            w = self.__mount.config.standort_laenge.signed_dms()
            sig = '-' if w[0] > 0 else '+'
            dd = str(int(w[1])).rjust(3, '0')
            mm = str(int(w[2])).rjust(2, '0')
            ans = sig + dd + "*" + mm
            return ans + "#"

        elif cmd == ":GR":     # RA ------------------------------------------
            #ts = skyfield.api.load.timescale()
            #rade = self.__mount.config.standort.at(ts.now()).from_altaz(alt_degrees=self.__mount.el.position.degrees,
            #                                                            az_degrees=self.__mount.az.position.degrees)
            #_ra, _dec, _distance = rade.radec()
            _ra, _dec, _distance = self.__mount.terminal.position.radec()
            w = _ra.signed_hms()
            hh = str(int(w[1])).rjust(2, '0')
            mm = str(int(w[2])).rjust(2, '0')
            ans = hh + ":" + mm
            if self.lx200_highpresitiondisplay:
                ss = str(int(w[3])).rjust(2, '0')
                ans += ':' + ss
            else:
                ans += '.' + str(int(w[3]/6))
            return ans + "#"

        elif cmd == ":Gt":     # Standort Breitenkreis. Nord ist Positiv ------------------------------------------
            w = self.__mount.config.standort_breite.signed_dms()
            sig = '+' if w[0] >= 0 else '-'
            dd = str(int(w[1])).rjust(2, '0')
            mm = str(int(w[2])).rjust(2, '0')
            ans = sig + dd + "*" + mm
            return ans + "#"

        elif cmd == ":GC":     # aktuelles lokales Datum ----------------------------------------------
            return time.strftime("%m/%d/%y#", time.localtime())

        elif cmd == ":GL":     # aktuelle lokale Uhrzeit ----------------------------------------------
            return time.strftime("%H:%M:%S#", time.localtime())

        elif cmd == ":GG":     # Zeitzone ----------------------------------------------
            z = time.strftime("%z", time.localtime())[:3]
            if z[0] == '+':
                z = z.replace('+', '-')
            else:
                z = z.replace('-', '+')
            return z + "#"

        elif cmd == ":GS":     # aktuelle Sternzeit ------------------------------------------
            ts = skyfield.api.load.timescale()
            rade = self.__mount.config.standort.at(ts.now()).from_altaz(alt_degrees=0, az_degrees=180)
            ra, __dec, __distance = rade.radec()
            w = ra.signed_hms()
            hh = str(int(w[1])).rjust(2, '0')
            mm = str(int(w[2])).rjust(2, '0')
            ss = str(int(w[3])).rjust(2, '0')
            ans = hh + ":" + mm + ":" + ss + "#"
            return ans

        elif cmd == ":GVD":    # Firmware Datum ------------------------------------------
            return "Jan 01 2019#"

        elif cmd == ":GVF":    # Version (LX200 sendet hier keine Antwort.) ------------------------------------------
            # return "ETX Autostar|A|43Eg|Apr 03 2007@11:25:53#"
            return "#"

        elif cmd == ":GVN":    # Firmware Nummer ------------------------------------------
            return "00.2#"

        elif cmd == ":GVP":    # Produktname ------------------------------------------
            return "Clipeum#"

        elif cmd == ":GVT":    # Firmware Zeit ------------------------------------------
            return "11:25:53#"

        elif cmd == ":GW":     # Alignement Status ------------------------------------------
            pass
            # todo

        elif cmd == ":GZ":     # AZ ------------------------------------------
            w = self.__mount.az.position.signed_dms()
            dd = str(int(w[1])).rjust(3, '0')
            mm = str(int(w[2])).rjust(2, '0')
            ans = dd + "*" + mm
            if self.lx200_highpresitiondisplay:
                ss = str(int(w[3])).rjust(2, '0')
                ans += '’' + ss
            return ans + "#"

        else:
            return 'CMD-Get-Fehler: ' + cmd

    def __lx200_move(self, cmd):
        if cmd == ":Me":       # Ost ------------------------------------------
            self.__mount.move_az_left()

        elif cmd == ":Mn":     # Nord ------------------------------------------
            self.__mount.move_el_up()

        elif cmd == ":Ms":     # Süd ------------------------------------------
            self.__mount.move_el_down()

        elif cmd == ":Mw":     # West ------------------------------------------
            self.__mount.move_az_right()

        elif cmd == ":MS":     # Zu Zielobjekt ------------------------------------------
            # todo
            return "0"

        else:
            return 'CMD-Move-Fehler: ' + cmd

        return ""

    def __lx200_stop(self, cmd):
        # todo
        if cmd == ":Q":       # Stopt alles ------------------------------------------
            self.__mount.stop()

        elif cmd == ":Qe":       # Ost ------------------------------------------
            self.__mount.stop_az()

        elif cmd == ":Qn":     # Nord ------------------------------------------
            self.__mount.stop_el()

        elif cmd == ":Qs":     # Süd ------------------------------------------
            self.__mount.stop_el()

        elif cmd == ":Qw":     # West ------------------------------------------
            self.__mount.stop_az()

        else:
            return 'CMD-Stop-Fehler: ' + cmd

        return ""

    def __lx200_rate(self, cmd):
        if cmd == ":RC":       # Zentrierung ------------------------------------------
            self.__mount.set_motorrate_manuell(False)

        elif cmd == ":RG":     # Nachführung ------------------------------------------
            self.__mount.set_motorrate_manuell(False)

        elif cmd == ":RM":     # Finden ------------------------------------------
            self.__mount.set_motorrate_manuell(True)

        elif cmd == ":RS":     # Schwenken (maximal) ------------------------------------
            self.__mount.set_motorrate_manuell(True)

        else:
            return 'CMD-Rate-Fehler: ' + cmd

        return ""


    def __lx200_set(self, cmd):
        if cmd[2] == "t":       # Latitude (Breitengrad) ------------------------------------
            g = float(cmd[3:6])
            m = float(cmd[7:])
            brt = g + (m / 60)
            self.__mount.config.set_standort(brt, self.__mount.config.standort_laenge.degrees)
            return "1"

        elif cmd[2] == "g":       # Longitude (Längengrad) ------------------------------------
            g = float(cmd[3:6])
            m = float(cmd[7:])
            lng = g + (m / 60)
            if lng > 180:
                lng -= 360  # Die Amis zählen hinten rum
            lng *= -1       # Vorzeichen ändern wegen der Amis
            self.__mount.config.set_standort(self.__mount.config.standort_breite.degrees, lng)
            return "1"

        elif cmd[2] == "r":       # Set Target RA ------------------------------------------
            hms = cmd[3:].split(':')
            h=int(hms[0])
            m=int(hms[1])
            s=float(hms[2].replace(',', '.'))
            self.__mount.target.set_target_tracking(skyfield.units.Angle(hours=(h, m, s)), self.__mount.target.ziel_de)
            return "1"

        elif cmd[2] == "d":       # Set Target DEC ------------------------------------------
            dms = cmd[3:].split('*')
            d=int(dms[0])
            ms=dms[1].split(':')
            m=int(ms[0])
            s=float(ms[1].replace(',', '.'))
            self.__mount.target.set_target_tracking(self.__mount.target.ziel_ra, skyfield.units.Angle(degrees=(d, m, s)))
            return "1"


    # Thread -----------------------------------------------------------------------------------------------------------
    def run(self):
        self.__app.run(host="0.0.0.0", port=self.__mount.config.serverport)


if __name__ == '__main__':
    import mount

    mnt = mount.Mount()
    server = Controlserver(mnt)
    print('Start Server')

    i = 0
    try:
        while True:
            i += 1
            server.x = i
            time.sleep(1)
            print(i)

    except KeyboardInterrupt:
        pass
