"""
Parameter, Einstellungen, ... werden in einer SQLite Datenbank gespeichert.
"""

from logger import log
import sqlite3


class Database(object):
    def __init__(self):

        try:
            self.sl3database = sqlite3.connect("./clipeum.sl3", check_same_thread=False)
        except Exception as e:
            log.error(e)
            print(e)

    def __del__(self):
        self.close()

    def close(self):
        try:
            self.sl3database.commit()
            self.sl3database.close()
        except sqlite3.DatabaseError:
            pass

    # Parameter --------------------------------------------
    def __get_parameter(self, parameter):
        try:
            sql = '''select parametervalue
                       from parameter
                      where parameterkey = :para'''
            p = {"para": parameter}
            c = self.sl3database.cursor()
            c.execute(sql, p)
            data = c.fetchone()
            return data

        except Exception as e:
            log.error(e)
            print(e)

    def get_parameter_str(self, parameter):
        data = self.__get_parameter(parameter)
        if data is None:
            return "?"
        else:
            return data[0]

    def get_parameter_int(self, parameter):
        data = self.__get_parameter(parameter)
        if data is None:
            return int(0)
        else:
            return int(float(data[0]))

    def get_parameter_float(self, parameter):
        data = self.__get_parameter(parameter)
        if data is None:
            return float(0.0)
        else:
            return float(data[0])

    def save_parameter(self, parameter, newvalue):
        try:
            sql = '''update parameter
                        set parametervalue = :newval
                      where parameterkey = :para'''
            p = {"para": parameter, "newval": newvalue}
            c = self.sl3database.cursor()
            c.execute(sql, p)
            self.sl3database.commit()

        except Exception as e:
            log.error(e)
            print(e)

    # Achsparameter --------------------------------------------
    def __get_achsparameter(self, achse, parameter):
        try:
            sql = '''select a.parametervalue
                       from achse a
                      where a.achse = :ax
                        and a.parameterkey = :para'''
            p = {"ax": achse, "para": parameter}
            c = self.sl3database.cursor()
            c.execute(sql, p)
            data = c.fetchone()
            return data

        except Exception as e:
            log.error(e)
            print(e)

    def get_achsparameter_str(self, achse, parameter):
        data = self.__get_achsparameter(achse, parameter)
        if data is None:
            return "?"
        else:
            return data[0]

    def get_achsparameter_int(self, achse, parameter):
        data = self.__get_achsparameter(achse, parameter)
        if data is None:
            return int(0)
        else:
            return int(float(data[0]))

    def get_achsparameter_float(self, achse, parameter):
        data = self.__get_achsparameter(achse, parameter)
        if data is None:
            return float(0.0)
        else:
            return float(data[0])

    def save_achsparameter(self, achse, parameter, newvalue):
        try:
            sql = '''update achse
                        set parametervalue = :newval
                      where achse = :ax
                        and parameterkey = :para'''
            p = {"ax": achse, "para": parameter, "newval": newvalue}
            c = self.sl3database.cursor()
            c.execute(sql, p)
            self.sl3database.commit()

        except Exception as e:
            log.error(e)
            print(e)

    # Zielpositionen --------------------------------------------
    def get_position(self, parameter):
        try:
            sql = '''select az, el, bezeichnung
                       from zielposition
                      where position = :para'''
            p = {"para": parameter}
            c = self.sl3database.cursor()
            c.execute(sql, p)
            data = c.fetchone()
            return data

        except Exception as e:
            log.error(e)
            print(e)

    def get_position_all(self):
        try:
            c = self.sl3database.cursor()
            c.execute('select position, az, el, bezeichnung, loeschbar from zielposition order by position')
            return c.fetchall()

        except Exception as e:
            log.error(e)
            print(e)

    def add_new_position(self):
        try:
            c = self.sl3database.cursor()
            maxposition = c.execute("select ifnull(max(position), 'z0')"
                                    " from zielposition where position like 'z%'").fetchone()
            maxnumber = int(maxposition[0][1:])
            nextposition = 'z' + str(maxnumber + 1).rjust(6, '0')
            sql = "insert into zielposition (position, az, el, bezeichnung, loeschbar) values (:pos, 180, 45, 'neu', 1)"
            p = {"pos": nextposition}
            c.execute(sql, p)
            self.sl3database.commit()

        except Exception as e:
            log.error(e)
            print(e)

    def delete_position(self, position):
        try:
            sql = '''delete from zielposition where position = :pos'''
            p = {"pos": position}
            c = self.sl3database.cursor()
            c.execute(sql, p)
            self.sl3database.commit()

        except Exception as e:
            log.error(e)
            print(e)

    def save_position(self, position, az, el, bezeichnung):
        try:
            sql = '''update zielposition
                        set az = :az
                          , el = :el
                          , bezeichnung = :bez
                      where position = :pos'''
            p = {"pos": position, "az": az, "el": el, "bez": bezeichnung}
            c = self.sl3database.cursor()
            c.execute(sql, p)
            self.sl3database.commit()

        except Exception as e:
            log.error(e)
            print(e)

    # Koordinatentransformation --------------------------------------------
    def __aenderung_referenzpunkt(self, dbcursor):
        dbcursor.execute("update parameter set parametervalue = 1 where parameterkey = '_neue_referenzpunkte_'")


    def get_transform_refpoint_all(self):
        try:
            c = self.sl3database.cursor()
            az = c.execute("select sensorvalue, winkel from referenzpunkt_az order by sensorvalue").fetchall()
            el = c.execute("select sensorvalue, winkel from referenzpunkt_el order by sensorvalue").fetchall()
            np = c.execute("select parametervalue from parameter "
                           "where parameterkey = '_neue_referenzpunkte_'").fetchone()
            return {'az': az, 'el': el, 'newrefpoint': np}

        except Exception as e:
            log.error(e)
            print(e)


    def add_new_transform_refpoint_az(self, sensorvalue, winkel):
        try:
            c = self.sl3database.cursor()
            sql = "insert or replace into referenzpunkt_az (sensorvalue, winkel) values (:s, :w)"
            p = {"s": sensorvalue, "w": round(winkel, 4)}
            c.execute(sql, p)
            self.__aenderung_referenzpunkt(c)
            self.sl3database.commit()

        except Exception as e:
            log.error(e)
            print(e)


    def add_new_transform_refpoint_el(self, sensorvalue, winkel):
        try:
            c = self.sl3database.cursor()
            sql = "insert or replace into referenzpunkt_el (sensorvalue, winkel) values (:s, :w)"
            p = {"s": sensorvalue, "w": round(winkel, 4)}
            c.execute(sql, p)
            self.__aenderung_referenzpunkt(c)
            self.sl3database.commit()

        except Exception as e:
            log.error(e)
            print(e)

    def delete_transform_refpoint_az(self, sensorvalue):
        try:
            sql = '''delete from referenzpunkt_az where sensorvalue = :s'''
            p = {"s": sensorvalue}
            c = self.sl3database.cursor()
            c.execute(sql, p)
            self.__aenderung_referenzpunkt(c)
            self.sl3database.commit()

        except Exception as e:
            log.error(e)
            print(e)

    def update_transform_refpoint_az(self, sensorvalue, winkel):
        try:
            sql = '''update referenzpunkt_az set winkel = :w where sensorvalue = :s'''
            p = {"s": sensorvalue, "w": round(winkel, 4)}
            c = self.sl3database.cursor()
            c.execute(sql, p)
            self.__aenderung_referenzpunkt(c)
            self.sl3database.commit()

        except Exception as e:
            log.error(e)
            print(e)

    def update_transform_refpoint_el(self, sensorvalue, winkel):
        try:
            sql = '''update referenzpunkt_el set winkel = :w where sensorvalue = :s'''
            p = {"s": sensorvalue, "w": round(winkel, 4)}
            c = self.sl3database.cursor()
            c.execute(sql, p)
            self.__aenderung_referenzpunkt(c)
            self.sl3database.commit()

        except Exception as e:
            log.error(e)
            print(e)

    def delete_transform_refpoint_el(self, sensorvalue):
        try:
            sql = '''delete from referenzpunkt_el where sensorvalue = :s'''
            p = {"s": sensorvalue}
            c = self.sl3database.cursor()
            c.execute(sql, p)
            self.__aenderung_referenzpunkt(c)
            self.sl3database.commit()

        except Exception as e:
            log.error(e)
            print(e)

    def reset_transform_newrefpoint(self):
        try:
            c = self.sl3database.cursor()
            c.execute("update parameter set parametervalue = 0 where parameterkey = '_neue_referenzpunkte_'")
            self.sl3database.commit()

        except Exception as e:
            log.error(e)
            print(e)

    def get_transform_linearity_divergence(self):
        try:
            c = self.sl3database.cursor()
            # AZ ---------------
            azbase = c.execute("""select min(sensorvalue),
                                        min(winkel),
                                        (max(winkel) - min(winkel)) / (count(*) - 1) as winkelschritt
                                    from transform_az""").fetchone()

            sql = """select round((winkel - :minwinkel) * :aufloesungprograd, 0) / :aufloesungprograd as winkelgruppe,
                            min(abweichung),
                            max(abweichung)
                    from (  select winkel,
                                    (winkel - :minwinkel) - ((sensorvalue - :minsensor) * :winkelschritt) as abweichung
                            from transform_az
                        )
                    group by round((winkel - :minwinkel) * :aufloesungprograd, 0) / :aufloesungprograd
                    order by 1 """
            p = {"minsensor": azbase[0], "minwinkel": azbase[1], "winkelschritt": azbase[2], "aufloesungprograd": 2}
            c = self.sl3database.cursor()
            c.execute(sql, p)
            az = c.fetchall()

            # EL ---------------
            elbase = c.execute("""select min(sensorvalue),
                                        min(winkel),
                                        (max(winkel) - min(winkel)) / (count(*) - 1) as winkelschritt
                                    from transform_el""").fetchone()

            sql = """select round((winkel - :minwinkel) * :aufloesungprograd, 0) / :aufloesungprograd as winkelgruppe,
                            min(abweichung),
                            max(abweichung)
                    from (  select winkel,
                                    (winkel - :minwinkel) - ((sensorvalue - :minsensor) * :winkelschritt) as abweichung
                            from transform_el
                        )
                    group by round((winkel - :minwinkel) * :aufloesungprograd, 0) / :aufloesungprograd
                    order by 1 """
            p = {"minsensor": elbase[0], "minwinkel": elbase[1], "winkelschritt": elbase[2], "aufloesungprograd": 4}
            c = self.sl3database.cursor()
            c.execute(sql, p)
            el = c.fetchall()

            return {'az': az, 'el': el}

        except Exception as e:
            log.error(e)
            print(e)

    # Referenzsterne --------------------------------------------
    def get_referenzstern_all(self):
        try:
            c = self.sl3database.cursor()
            c.execute('select stern, name, bayername from referenzstern order by name')
            return c.fetchall()

        except Exception as e:
            log.error(e)
            print(e)

    def get_referenzstern_position(self, stern):
        try:
            sql = '''select ra, dec
                       from referenzstern
                      where stern = :star'''
            p = {"star": stern}
            c = self.sl3database.cursor()
            c.execute(sql, p)
            data = c.fetchone()
            return {"ra": float(data[0]),
                    "dec": float(data[1])}

        except Exception as e:
            log.error(e)
            print(e)

    # Nachführintervalle --------------------------------------------
    def get_intervall(self, parameter):
        try:
            sql = '''select intervall
                       from intervall
                      where objekt = :para'''
            p = {"para": parameter}
            c = self.sl3database.cursor()
            c.execute(sql, p)
            data = c.fetchone()
            if data is None:
                return 1    # Standardintervall eine Sekunde, wenn Parameter nicht vorhanden
            else:
                return int(data[0])

        except Exception as e:
            log.error(e)
            print(e)

    def get_intervall_all(self):
        try:
            c = self.sl3database.cursor()
            c.execute('select objekt, intervall, bezeichnung from intervall order by bezeichnung')
            return c.fetchall()

        except Exception as e:
            log.error(e)
            print(e)

    def save_intervall(self, objekt, newvalue):
        try:
            sql = '''update intervall
                        set intervall = :newval
                      where objekt = :obj'''
            p = {"obj": objekt, "newval": newvalue}
            c = self.sl3database.cursor()
            c.execute(sql, p)
            self.sl3database.commit()

        except Exception as e:
            log.error(e)
            print(e)


if __name__ == '__main__':
    db = Database()

    print("Zielpositionen:")
    print("astra", db.get_position("astra"))
    print("wartung", db.get_position("wartung"))
    print("park", db.get_position("park"))
    print("not", db.get_position("not"))
    print("?", db.get_position("?"))
    print("Intervall Sun", db.get_intervall("sun"))
    print("Intervall ?", db.get_intervall("?"))
