"""
Analogmotoren, Ansteuerung mittels Schalter
"""

from logger import log
import database
import interface
from motor import Motor
from threading import Thread
import time


class MotorAnalog(Motor, Thread):
    def __init__(self, achse, db, interface):
        Thread.__init__(self)
        Motor.__init__(self, achse, db, interface)

        self.motortyp = self._interface.MOTORTYP_ANALOG
        self.__istRate = self.MOTOR_STOP
        self.__sollRate = self.MOTOR_STOP
        self.__istDir = -99
        self.__sollDir = self.MOTOR_RIGHT
        self.__isOn = False
        self.__sollEin = False

        Motor._set_motortype2interface(self)

        self.__threadIsRunning = True
        self.start()

    def __del__(self):
        self.close()

    def close(self):
        self.emergency_stop()
        if self.__threadIsRunning:
            self.__threadIsRunning = False
            self.join(1)

    @property
    def is_on(self):
        return self.__isOn

    @property
    def direction_ist(self):
        return self.__istDir if self.__isOn else self.MOTOR_STOP

    @property
    def rate_ist(self):
        return self.__istRate if self.__isOn else self.MOTOR_STOP

    @property
    def rate_soll(self):
        return self.__sollRate

    def set_direction(self, direction):
        Motor.set_direction(self, direction)
        self.__sollDir = direction

    def set_rate(self, rate):
        Motor.set_rate(self, rate)
        self.__sollRate = rate
        if rate == self.MOTOR_STOP:
            self.__sollEin = False
        else:
            self.__sollEin = True

    def set_motor_is_blocked_time(self, blockedtime):
        pass

    def emergency_stop(self):
        self.__sollRate = self.MOTOR_STOP
        self.__sollEin = False
        self._interface.set_motorpower(self._achse, False)
        self.__isOn = False

    # Thread --------------------------------------------------
    def run(self):
        while self.__threadIsRunning:
            time.sleep(0.025)

            # Motor ist ein
            if self.__isOn:
                # SOLL weicht vom IST ab
                if self.__istDir != self.__sollDir \
                        or self.__istRate != self.__sollRate \
                        or not self.__sollEin:
                    # Motor ausschalten
                    self._interface.set_motorpower(self._achse, False)
                    self.__isOn = False
                    time.sleep(0.2)

            # Motor ist aus --> Richtung und Rate umschalten
            else:
                if self.__sollEin:
                    # Richtung weicht ab
                    if self.__istDir != self.__sollDir:
                        # Richtung schalten
                        self._interface.set_motordirection(self._achse, (self.__sollDir == self.MOTOR_RIGHT))
                        self.__istDir = self.__sollDir
                    # Rate weicht ab
                    if self.__istRate != self.__sollRate:
                        # Rate schalten
                        self._interface.set_motorspeed(self._achse, (self.__sollRate == self.MOTOR_FAST))
                        self.__istRate = self.__sollRate

                    time.sleep(0.2)
                    # Motor einschalten
                    self._interface.set_motorpower(self._achse, True)
                    self.__isOn = True


if __name__ == '__main__':
    import configuration

    db = database.Database()
    config = configuration.Configuration(db)
    iface = interface.Interface(config)

    for i in ('A'):
        a = MotorAnalog(i, db, iface)
        print("Achse:", i)
        print("Motortyp:", a.motortyp)

        a.set_direction(a.MOTOR_LEFT)
        a.set_rate(a.MOTOR_FAST)
        print("left fast")
        time.sleep(5)
        a.set_rate(a.MOTOR_SLOW)
        print("left slow")
        time.sleep(5)
        a.set_direction(a.MOTOR_RIGHT)
        print("right slow")
        time.sleep(5)
        a.set_direction(a.MOTOR_LEFT)
        a.set_rate(a.MOTOR_FAST)
        print("left fast")
        time.sleep(5)
        a.set_rate(a.MOTOR_STOP)
        print("STOP")


    a.close()
    iface.close()
    config.close()
    db.close()
