"""
Schrittmotoren
"""

from logger import log
import database
import interface
from motor import Motor
from threading import Thread
import time
import math


class MotorStepper(Motor, Thread):
    LEVER_FAST = 13
    LEVER_SLOW = 5
    LEVER_STOP = 0

    def __init__(self, achse, db, interface):
        Thread.__init__(self)
        Motor.__init__(self, achse, db, interface)

        self.motortyp = self._interface.MOTORTYP_STEPPER
        self.__isOn = False
        self.__motorLeverIst = 0
        self.__motorLeverIstMax = self.LEVER_FAST   # Maximale Drehrate
        self.__motorLeverSoll = 0
        self.__motorLeverSollSign = 0
        self.__steppermotorMaxFrequency = self._db.get_achsparameter_int(self._achse, "motormaxfreq")
        self.__sensorBlockedMotorWaitToSync = self._db.get_achsparameter_int(self._achse, "sensorsyncdelay")

        Motor._set_motortype2interface(self)

        self.__threadIsRunning = True
        self.start()

    def __del__(self):
        self.close()

    def close(self):
        self.emergency_stop()
        if self.__threadIsRunning:
            self.__threadIsRunning = False
            self.join(0.5)

    @property
    def is_on(self):
        return self.__isOn

    @property
    def direction_ist(self):
        return (self.__motorLeverIst > 0) - (self.__motorLeverIst < 0)      # Entspricht sign()

    @property
    def rate_ist(self):
        __ar = abs(self.__motorLeverIst)
        if __ar > self.LEVER_SLOW:
            __r = self.MOTOR_FAST
        elif __ar > self.LEVER_STOP:
            __r = self.MOTOR_SLOW
        else:
            __r = self.MOTOR_STOP
        return __r

    @property
    def rate_soll(self):
        __ar = abs(self.__motorLeverSoll)
        if __ar > self.LEVER_SLOW:
            __r = self.MOTOR_FAST
        elif __ar > self.LEVER_STOP:
            __r = self.MOTOR_SLOW
        else:
            __r = self.MOTOR_STOP
        return __r

    def __set_motorrate(self):
        mli = abs(self.__motorLeverIst)
        if mli == 0:
            pass
        elif mli == 1:
            self._interface.set_steppermotorspeed(self._achse, 0)
            self.__isOn = False
        elif mli == 2:
            self._interface.set_steppermotorspeed(self._achse, self.__steppermotorMaxFrequency / 20.0)
            self.__isOn = True
        elif mli == 3:
            self._interface.set_steppermotorspeed(self._achse, self.__steppermotorMaxFrequency / 10.0)
        elif mli == 4:
            self._interface.set_steppermotorspeed(self._achse, self.__steppermotorMaxFrequency / 5.00)
        elif mli == 5:
            self._interface.set_steppermotorspeed(self._achse, self.__steppermotorMaxFrequency / 3.33)
        elif mli == 6:
            self._interface.set_steppermotorspeed(self._achse, self.__steppermotorMaxFrequency / 2.50)
        elif mli == 7:
            self._interface.set_steppermotorspeed(self._achse, self.__steppermotorMaxFrequency / 2.00)
        elif mli == 8:
            self._interface.set_steppermotorspeed(self._achse, self.__steppermotorMaxFrequency / 1.67)
        elif mli == 9:
            self._interface.set_steppermotorspeed(self._achse, self.__steppermotorMaxFrequency / 1.43)
        elif mli == 10:
            self._interface.set_steppermotorspeed(self._achse, self.__steppermotorMaxFrequency / 1.25)
        elif mli == 11:
            self._interface.set_steppermotorspeed(self._achse, self.__steppermotorMaxFrequency / 1.11)
        elif mli == 12:
            self._interface.set_steppermotorspeed(self._achse, self.__steppermotorMaxFrequency)

    def __set_motordirection(self):
        if self.__motorLeverIst == -1:
            self._interface.set_motordirection(self._achse, False)
        elif self.__motorLeverIst == 0:
            pass
        elif self.__motorLeverIst == 1:
            self._interface.set_motordirection(self._achse, True)

    def set_direction(self, direction):
        Motor.set_direction(self, direction)
        self.__motorLeverSollSign = direction
        self.__motorLeverSoll = math.copysign(self.__motorLeverSoll, direction)

    def set_rate(self, rate):
        Motor.set_rate(self, rate)
        if rate == self.MOTOR_FAST:
            __rate = self.LEVER_FAST
        elif rate == self.MOTOR_SLOW:
            __rate = self.LEVER_SLOW
        else:
            __rate = self.LEVER_STOP

        self.__motorLeverSoll = math.copysign(__rate, self.__motorLeverSollSign)

    def set_motor_is_blocked_time(self, blockedtime):
        # log.info('BLOCKTIME: ' + str(blockedtime) + " " + str(self.__sensorBlockedMotorWaitToSync))

        if blockedtime <= (self.__sensorBlockedMotorWaitToSync / 5):        # Im ersten 1/5 der Zeit nicht reagieren
            self.__motorLeverIstMax = self.LEVER_FAST
        else:                                                               # die restliche Zeit herunerregeln.
            _wt = self.__sensorBlockedMotorWaitToSync * (4 / 5)
            _bt = blockedtime - (self.__sensorBlockedMotorWaitToSync / 5)
            _timetoendpct = 1 - (_bt / _wt)
            _lever = int(math.ceil(self.LEVER_FAST * _timetoendpct))
            _lever = max(_lever, 2)
            _lever = min(_lever, self.LEVER_FAST)
            self.__motorLeverIstMax = _lever

            # log.info('LEVERMAX: ' + ' ' + str(_wt) + ' ' + str(_timetoendpct) + ' ' + str(self.__motorLeverIstMax))
        # log.info('LEVERMAX: ' + str(self.__motorLeverIstMax) + " " + str(self.__motorLeverIst))

    def emergency_stop(self):
        self.__motorLeverSoll = 0
        self.__motorLeverIst = 0
        self._interface.set_steppermotorspeed(self._achse, 0)
        self.__isOn = False

    # Thread --------------------------------------------------
    def run(self):
        while self.__threadIsRunning:
            if self.__motorLeverIst == self.__motorLeverSoll:
                time.sleep(0.2)
            else:
                time.sleep(0.04)        # Änderung 13.7.2019 langsamerer Anlauf

            if self.__motorLeverIst < self.__motorLeverSoll:
                self.__motorLeverIst = self.__motorLeverIst + 1
            if self.__motorLeverIst > self.__motorLeverSoll:
                self.__motorLeverIst = self.__motorLeverIst - 1

            # Drehrate bei blockierenden Motor verringern
            if self.__motorLeverIst > 0:
                if self.__motorLeverIst > self.__motorLeverIstMax:      # Maximale Drehrate
                    self.__motorLeverIst = self.__motorLeverIstMax
            else:
                if self.__motorLeverIst < -self.__motorLeverIstMax:      # Maximale Drehrate
                    self.__motorLeverIst = -self.__motorLeverIstMax

            self.__set_motordirection()
            self.__set_motorrate()


if __name__ == '__main__':
    import configuration

    db = database.Database()
    config = configuration.Configuration(db)
    iface = interface.Interface(config)

    for i in ('E'):
        a = MotorStepper(i, db, iface)
        print("Achse:", i)
        print("Motortyp:", a.motortyp)

        #print("SensorN:", a.ttt)
        #time.sleep(3)
        #print("SensorS:", a.ttt)

        a.set_direction(a.MOTOR_LEFT)
        a.set_rate(a.MOTOR_FAST)

        time.sleep(4)
        a.set_direction(a.MOTOR_RIGHT)
        time.sleep(8)



    a.close()
    iface.close()
    config.close()
    db.close()

