
from logger import log
from threading import Thread
import time
import database
import configuration
import interface
import sensor_toangle
import achse
import motor
import terminal
import controlserver
import target


class ExitClipeum(Exception):
    def __init__(self):
        pass


class Mount(Thread):
    def __init__(self):
        Thread.__init__(self)
        self.exit_clipeum = False
        self.__threadIsRunning = False

        self.db = database.Database()
        self.config = configuration.Configuration(self.db)

        self.iface = interface.Interface(self.config)
        self.sensor2angle = sensor_toangle.Sensor2Angle(self.db)

        self.az = achse.Achse(achse.Achse.ACHSE_AZ, self.db, self.iface, self.sensor2angle)
        self.el = achse.Achse(achse.Achse.ACHSE_EL, self.db, self.iface, self.sensor2angle)
        self.server = controlserver.Controlserver(self)
        self.terminal = terminal.Terminal(self)
        self.target = target.Target(self)

        self.sollRateManuell = motor.Motor.MOTOR_FAST

        # Thread starten
        self.__threadIsRunning = True
        self.start()

    def __del__(self):
        self.close()

    def close(self):
        if self.__threadIsRunning:
            self.__threadIsRunning = False
            self.join(1)

        self.stop()
        self.target.close()
        self.terminal.close()
        self.server.close()
        self.az.close()
        self.el.close()
        self.sensor2angle.close()
        self.iface.close()
        self.config.close()
        self.db.close()

    def stop(self):
        self.target.set_target(None)
        self.az.stop()
        self.el.stop()

    def stop_az(self):
        self.target.set_target(None)
        self.az.stop()

    def stop_el(self):
        self.target.set_target(None)
        self.el.stop()

    def set_motorrate_manuell(self, fast):
        self.target.set_target(None)
        self.sollRateManuell = motor.Motor.MOTOR_FAST if fast else motor.Motor.MOTOR_SLOW
        if self.az.motor_rate_soll > motor.Motor.MOTOR_STOP:
            self.az.set_rate_manuell(self.sollRateManuell)
        if self.el.motor_rate_soll > motor.Motor.MOTOR_STOP:
            self.el.set_rate_manuell(self.sollRateManuell)

    def move_az_right(self, stop_if_moving=False):
        self.target.set_target(None)
        if stop_if_moving and self.az.motor_rate_ist > motor.Motor.MOTOR_STOP:    # Bewegt sich der Motor noch
            self.stop_az()                                                      # dann stoppen
        else:
            self.az.set_direction_manuell(motor.Motor.MOTOR_RIGHT)
            self.az.set_rate_manuell(self.sollRateManuell)

    def move_az_left(self, stop_if_moving=False):
        self.target.set_target(None)
        if stop_if_moving and self.az.motor_rate_ist > motor.Motor.MOTOR_STOP:    # Bewegt sich der Motor noch
            self.stop_az()                                                      # dann stoppen
        else:
            self.az.set_direction_manuell(motor.Motor.MOTOR_LEFT)
            self.az.set_rate_manuell(self.sollRateManuell)

    def move_el_up(self, stop_if_moving=False):
        self.target.set_target(None)
        if stop_if_moving and self.el.motor_rate_ist > motor.Motor.MOTOR_STOP:    # Bewegt sich der Motor noch
            self.stop_el()                                                      # dann stoppen
        else:
            self.el.set_direction_manuell(motor.Motor.MOTOR_RIGHT)
            self.el.set_rate_manuell(self.sollRateManuell)

    def move_el_down(self, stop_if_moving=False):
        self.target.set_target(None)
        if stop_if_moving and self.el.motor_rate_ist > motor.Motor.MOTOR_STOP:    # Bewegt sich der Motor noch
            self.stop_el()                                                      # dann stoppen
        else:
            self.el.set_direction_manuell(motor.Motor.MOTOR_LEFT)
            self.el.set_rate_manuell(self.sollRateManuell)

    def move_to(self, az, el):
        self.az.move_to_target(az)
        self.el.move_to_target(el)

    # Thread -----------------------
    def run(self):
        while self.__threadIsRunning:
            time.sleep(0.25)


if __name__ == '__main__':

    log.info('*** End Clipeum ***')
