"""
Zielobjekt
"""

from logger import log
from threading import Thread
import time
import skyfield
from skyfield import api, units


class Target(Thread):
    TARGETACHSENMODUS_ALTAZ = "A"
    TARGETACHSENMODUS_RADEC = "R"

    TARGET_PARK = "park"
    TARGET_MAINTAIN = "wartung"
    # TARGET_EMERGENCY = "not"
    TARGET_SUN = "sun"
    TARGET_MOON = "moon"
    TARGET_JUPITER = "jupiter"
    TARGET_TRACKING = "track"

    def __init__(self, mount):
        Thread.__init__(self)
        self.__mount = mount

        self.ziel_ra = skyfield.units.Angle(hours=0)
        self.ziel_de = skyfield.units.Angle(degrees=0)
        self.ziel_az = skyfield.units.Angle(degrees=0)
        self.ziel_el = skyfield.units.Angle(degrees=0)

        self.target_achsen_modus = None
        self.target_name = None
        self.target = None
        self.target_nachtposition = False

        self.trackingpoint = 0
        self.sun = None
        self.moon = None
        self.jupiter = None

        # Thread starten
        self.__threadIsRunning = True
        self.start()

    def __del__(self):
        self.close()

    def close(self):
        if self.__threadIsRunning:
            self.__threadIsRunning = False
            self.join(1)

    def set_target_altaz(self, az, el):
        self.set_target(None)
        self.ziel_az = az
        self.ziel_el = el
        self.target_achsen_modus = self.TARGETACHSENMODUS_ALTAZ
        self.__mount.move_to(self.ziel_az, self.ziel_el)

    def set_target_tracking(self, ra, de):
        self.set_target(self.TARGET_TRACKING)
        self.ziel_ra = ra
        self.ziel_de = de

    def set_target(self, target):
        log.info("Ziel: " + str(target))

        # if self.target != target:
        #     log.info("STOP: " + str(target))
        #     self.__mount.stop()

        if self.target is not None and target is None:  #Beim entfernen eines Ziels, die Montierung stoppen.
            self.__mount.az.stop()
            self.__mount.el.stop()

        self.target = target

        if target is None:
            self.__clear_target()

        elif target in (self.TARGET_SUN,
                        self.TARGET_MOON,
                        self.TARGET_JUPITER,
                        self.TARGET_TRACKING):
            self.target_achsen_modus = self.TARGETACHSENMODUS_RADEC

        else:
            _t = self.__mount.db.get_position(target)
            if _t is None:
                self.__clear_target()
            else:
                self.target_name = _t[2]
                self.target_nachtposition = False
                self.ziel_az = skyfield.units.Angle(degrees=_t[0])
                self.ziel_el = skyfield.units.Angle(degrees=_t[1])
                self.target_achsen_modus = self.TARGETACHSENMODUS_ALTAZ
                self.__mount.move_to(self.ziel_az, self.ziel_el)

    def __clear_target(self):
        self.target_achsen_modus = None
        self.target_name = None
        self.target = None
        self.target_nachtposition = False

    def __next_trackingpoint(self, now, obj):
        trackingintervall = self.__mount.db.get_intervall(obj) / 86400  # Intervall in Tagen
        self.trackingpoint = now + trackingintervall  # Neuer Trackingzeitpunkt ein Intervall in die Zukunft setzen

        # Zielposition zwischen jetzt un ein Intervall in Zukunft
        th = now + (trackingintervall / 2)  # + halbes Intervall
        t = self.__mount.config.timescale.tt_jd(th)  # Time-Objekt halbes Intervall in der Zukunft
        return t

    def __check_folge_bis_horizonthoehe(self):
        """Folge einem Objekt bis x° über/unter Horizonthöhe.
           Unterschreitet das Zielobjekt die angegebene Höhe wird die Montierung in die Nachtposition gefahren.
        """

        if self.ziel_el.degrees < self.__mount.config.nachtpositionschwelle:
            _t = self.__mount.db.get_position('night')
            self.ziel_az = skyfield.units.Angle(degrees=_t[0])
            self.ziel_el = skyfield.units.Angle(degrees=_t[1])
            self.target_nachtposition = True
        else:
            self.target_nachtposition = False

    def __tracking_tracking(self, now):
        if self.trackingpoint <= now:
            t = self.__next_trackingpoint(now, 'tracking')
            trackingstar = skyfield.api.Star(ra=self.ziel_ra, dec=self.ziel_de)
            astrometric = self.__mount.config.standort.at(t).observe(trackingstar)
            self.ziel_el, self.ziel_az, __dis = astrometric.apparent().altaz()
            self.__check_folge_bis_horizonthoehe()
            self.__mount.move_to(self.ziel_az, self.ziel_el)

    def __tracking_sun(self, now):
        if self.sun is None:
            self.sun = self.__mount.config.planets['sun']

        if self.trackingpoint <= now:
            t = self.__next_trackingpoint(now, 'sun')
            astrometric = self.__mount.config.standort.at(t).observe(self.sun)
            self.ziel_ra, self.ziel_de, __dis = astrometric.radec()
            self.ziel_el, self.ziel_az, __dis = astrometric.apparent().altaz()
            self.__check_folge_bis_horizonthoehe()
            self.__mount.move_to(self.ziel_az, self.ziel_el)

    def __tracking_moon(self, now):
        if self.moon is None:
            self.moon = self.__mount.config.planets['moon']

        if self.trackingpoint <= now:
            t = self.__next_trackingpoint(now, 'moon')
            astrometric = self.__mount.config.standort.at(t).observe(self.moon)
            self.ziel_ra, self.ziel_de, __dis = astrometric.radec()
            self.ziel_el, self.ziel_az, __dis = astrometric.apparent().altaz()
            self.__check_folge_bis_horizonthoehe()
            self.__mount.move_to(self.ziel_az, self.ziel_el)

    def __tracking_jupiter(self, now):
        if self.jupiter is None:
            self.jupiter = self.__mount.config.planets['JUPITER BARYCENTER']

        if self.trackingpoint <= now:
            t = self.__next_trackingpoint(now, 'jupiter')
            astrometric = self.__mount.config.standort.at(t).observe(self.jupiter)
            self.ziel_ra, self.ziel_de, __dis = astrometric.radec()
            self.ziel_el, self.ziel_az, __dis = astrometric.apparent().altaz()
            self.__check_folge_bis_horizonthoehe()
            self.__mount.move_to(self.ziel_az, self.ziel_el)

    # Thread -----------------------
    def run(self):
        oldtarget = '?'
        time.sleep(1)

        # Startmodus aktivieren
        if self.__mount.config.startmodus in (self.TARGET_SUN,
                                              self.TARGET_MOON,
                                              self.TARGET_JUPITER):
            self.set_target(self.__mount.config.startmodus)

        while self.__threadIsRunning:
            time.sleep(1)

            now = self.__mount.config.timescale.now().tt
            if oldtarget != self.target:
                self.trackingpoint = now
                oldtarget = self.target

            if self.target == self.TARGET_TRACKING:
                self.__tracking_tracking(now)

            elif self.target == self.TARGET_SUN:
                self.__tracking_sun(now)

            elif self.target == self.TARGET_MOON:
                self.__tracking_moon(now)

            elif self.target == self.TARGET_JUPITER:
                self.__tracking_jupiter(now)
