"""
Telemetrieanzeige auf Konsolen
"""


from logger import log
from threading import Thread
import time
import urllib
import urllib.request
import skyfield
from skyfield import api, units
from asciimatics.screen import Screen
from asciimatics.event import KeyboardEvent


class Terminal(Thread):
    def __init__(self, mount):
        Thread.__init__(self)

        self.__screen = Screen.open()
        self.__mount = mount

        self.__localhost_lx200 = "http://127.0.0.1:" + str(mount.config.serverport) + "/lx200/"
        self.__ts = skyfield.api.load.timescale()

        self.pos_ra = ""
        self.pos_de = ""
        self.pos_az = ""
        self.pos_el = ""
        self.position = ""
        self.ziel_ra = ""
        self.ziel_de = ""
        self.ziel_az = ""
        self.ziel_el = ""
        self.laengengrad = ""
        self.breitengrad = ""

        self.__old_ratecompass = (0, 0)
        self.__threadIsRunning = True
        self.start()

    def __del__(self):
        self.close()

    def close(self):
        if self.__threadIsRunning:
            self.__threadIsRunning = False
            self.join(1)
        self.__screen.close()
        print("QUIT CLIPEUM - STOPPING MOUNT - PLEASE WAIT")

    def __keypress(self, key):
        if key in (113, 81):    # q, Q
            self.__mount.target.set_target(None)
            __ = urllib.request.urlopen(self.__localhost_lx200 + "quit")

        elif key in (53, 350):   # 5, 5[NUM]
            self.__mount.stop()

        elif key in (54, -205):  # 6, Pfeil-rechts
            self.__mount.move_az_right(True)

        elif key in (52, -203):   # 4, Pfeil-links
            self.__mount.move_az_left(True)

        elif key in (56, -204):  # 8, Pfeil-oben
            self.__mount.move_el_up(True)

        elif key in (50, -206):   # 2, Pfeil-unten
            self.__mount.move_el_down(True)

        elif key == 43:             # +
            self.__mount.set_motorrate_manuell(True)

        elif key == 45:             # -
            self.__mount.set_motorrate_manuell(False)

        elif key in (44, -102, 46):  # "," ","[NUM], "."
            self.__mount.target.set_target(self.__mount.target.TARGET_MAINTAIN)

        elif key in (42, 116, 84):  # *, t, T
            rade = self.__mount.config.standort.at(
                self.__ts.now()).from_altaz(alt_degrees=self.__mount.el.position.degrees,
                                            az_degrees=self.__mount.az.position.degrees)
            _ra, _dec, _distance = rade.radec()
            self.__mount.target.set_target_tracking(_ra, _dec)

        elif key in (115, 83):      # s, S
            self.__mount.target.set_target(self.__mount.target.TARGET_SUN)

        elif key in (109, 77):      # m, M
            self.__mount.target.set_target(self.__mount.target.TARGET_MOON)

        elif key in (106, 74):      # j, J
            self.__mount.target.set_target(self.__mount.target.TARGET_JUPITER)

        elif key in (48, -101):      # 0, 0[NUM]
            self.__mount.target.set_target(self.__mount.target.TARGET_PARK)

    def __refresh_static_text(self):
        __m = "  " + self.__mount.config.name + "  "
        __m = __m.center(45, '*')
        self.__screen.print_at(__m, 0, 0, bg=Screen.COLOUR_BLACK,
                               colour=Screen.COLOUR_GREEN, attr=Screen.A_REVERSE)

        self.__screen.print_at("Position:", 0, 2, colour=Screen.COLOUR_WHITE)
        self.__screen.print_at("RA:", 10, 2)
        self.__screen.print_at("DE:", 29, 2)
        self.__screen.print_at("AZ:", 10, 3)
        self.__screen.print_at("EL:", 29, 3)

        self.__screen.print_at("Ziel:", 0, 5)
        self.__screen.print_at("RA:", 10, 5)
        self.__screen.print_at("DE:", 29, 5)
        self.__screen.print_at("AZ:", 10, 6)
        self.__screen.print_at("EL:", 29, 6)

        self.__screen.print_at("Modus:", 0, 8)
        self.__screen.print_at("UTC:", 0, 10)

        self.__screen.print_at("Standort:", 0, 11)
        self.__screen.print_at("Lng:", 10, 11)
        self.__screen.print_at("Brt:", 29, 11)

        # w = self.__mount.config.standort_laenge.signed_dms()
        # ow = 'E' if w[0] > 0 else 'W'
        # dd = str(int(w[1])).rjust(3, ' ')
        # mm = str(int(w[2])).rjust(2, '0')
        # self.laengengrad = dd + "° " + mm + "' " + ow
        # self.__screen.print_at(self.laengengrad, 15, 11, colour=Screen.COLOUR_WHITE)

        # w = self.__mount.config.standort_breite.signed_dms()
        # ns = 'N' if w[0] >= 0 else 'S'
        # dd = str(int(w[1])).rjust(2, ' ')
        # mm = str(int(w[2])).rjust(2, '0')
        # self.breitengrad = dd + "° " + mm + "' " + ns
        # self.__screen.print_at(self.breitengrad, 35, 11, colour=Screen.COLOUR_WHITE)

        self.__screen.print_at("Sensor:", 0, 13)
        self.__screen.print_at("AZ:", 10, 13)
        self.__screen.print_at("EL:", 29, 13)

        self.__screen.print_at("o", 27, 4, colour=Screen.COLOUR_BLUE, attr=Screen.A_BOLD)

    def __display_ratecompass(self):
        # AZ
        _az_rate = self.__mount.az.motor_rate_ist * self.__mount.az.motor_direction_ist
        if self.__old_ratecompass[0] != _az_rate:
            __az = ""
            if _az_rate <= -2:
                __az = "--o  "
            elif _az_rate <= -1:
                __az = " -o  "
            elif _az_rate <= 0:
                __az = "  o  "
            elif _az_rate <= 1:
                __az = "  o- "
            elif _az_rate <= 2:
                __az = "  o--"
            self.__screen.print_at(__az, 25, 4, colour=Screen.COLOUR_RED, attr=Screen.A_BOLD)
            self.__screen.print_at("o", 27, 4, colour=Screen.COLOUR_BLUE, attr=Screen.A_BOLD)
        # EL
        _el_rate = self.__mount.el.motor_rate_ist * self.__mount.el.motor_direction_ist
        if self.__old_ratecompass[1] != _el_rate:
            # alten Text löschen
            if self.__old_ratecompass[1] > 0:
                self.__screen.print_at(" ", 27, 4 - 1)
            if self.__old_ratecompass[1] < 0:
                self.__screen.print_at(" ", 27, 4 + 1)

            __el = ":"
            if _el_rate == 1:
                __el = "."
            if _el_rate == -1:
                __el = "'"
            if _el_rate > 0:
                self.__screen.print_at(__el, 27, 4 - 1, colour=Screen.COLOUR_RED, attr=Screen.A_BOLD)
            if _el_rate < 0:
                self.__screen.print_at(__el, 27, 4 + 1, colour=Screen.COLOUR_RED, attr=Screen.A_BOLD)

        self.__old_ratecompass = (_az_rate, _el_rate)

    # Thread -----------------------------------------------------------------------------------------------------------
    def run(self):
        self.__refresh_static_text()
        __oldtarget = ""
        __old_targetmodus = '?'
        __old_laengengrad = '?'
        __old_breitengrad = '?'

        while self.__threadIsRunning:
            # Eingabe ###############################################
            key = self.__screen.get_event()
            if type(key) == KeyboardEvent:
                log.debug("KEY: " + str(key.key_code))
                self.__keypress(key.key_code)

            # Ausgabe ###############################################
            time.sleep(0.3)
            # Position ------------------------------------------------------------------
            self.position = self.__mount.config.standort.at(
                self.__ts.now()).from_altaz(alt_degrees=self.__mount.el.position.degrees,
                                            az_degrees=self.__mount.az.position.degrees)
            _ra, _dec, _distance = self.position.radec()
            # RA
            w = _ra.signed_hms()
            sig = '+' if w[0] >= 0 else '-'
            hh = str(int(w[1])).rjust(2, ' ')
            mm = str(int(w[2])).rjust(2, '0')
            ss = str(int(w[3])).rjust(2, '0')
            self.pos_ra = sig + hh + "h " + mm + "' " + ss + '"'
            self.__screen.print_at(self.pos_ra, 14, 2, colour=Screen.COLOUR_WHITE)
            # DE
            w = _dec.signed_dms()
            sig = '+' if w[0] >= 0 else '-'
            dd = str(int(w[1])).rjust(2, ' ')
            mm = str(int(w[2])).rjust(2, '0')
            ss = str(int(w[3])).rjust(2, '0')
            self.pos_de = sig + dd + "° " + mm + "' " + ss + '"'
            self.__screen.print_at(self.pos_de, 33, 2, colour=Screen.COLOUR_WHITE)
            # AZ
            w = self.__mount.az.position.signed_dms()
            dd = str(int(w[1])).rjust(3, ' ')
            mm = str(int(w[2])).rjust(2, '0')
            ss = str(int(w[3])).rjust(2, '0')
            self.pos_az = dd + "° " + mm + "' " + ss + '"'
            c = Screen.COLOUR_RED if self.__mount.az.grenzverletzung else Screen.COLOUR_WHITE
            self.__screen.print_at(self.pos_az, 14, 3, colour=c)
            # EL
            w = self.__mount.el.position.signed_dms()
            sig = '+' if w[0] >= 0 else '-'
            dd = str(int(w[1])).rjust(2, ' ')
            mm = str(int(w[2])).rjust(2, '0')
            ss = str(int(w[3])).rjust(2, '0')
            self.pos_el = sig + dd + "° " + mm + "' " + ss + '"'
            c = Screen.COLOUR_RED if self.__mount.el.grenzverletzung else Screen.COLOUR_WHITE
            self.__screen.print_at(self.pos_el, 33, 3, colour=c)

            # Ziel ------------------------------------------------------------------
            if __old_targetmodus != self.__mount.target.target_achsen_modus:  # Bei Moduswechsel Ziele entfernen
                self.__screen.print_at("            ", 14, 5, colour=Screen.COLOUR_WHITE)
                self.__screen.print_at("            ", 33, 5, colour=Screen.COLOUR_WHITE)
                self.__screen.print_at("            ", 14, 6, colour=Screen.COLOUR_WHITE)
                self.__screen.print_at("            ", 33, 6, colour=Screen.COLOUR_WHITE)
                self.ziel_ra = self.ziel_de = self.ziel_az = self.ziel_el = ""
                __old_targetmodus = self.__mount.target.target_achsen_modus

            if self.__mount.target.target_achsen_modus == self.__mount.target.TARGETACHSENMODUS_RADEC:
                # RA
                w = self.__mount.target.ziel_ra.signed_hms()
                sig = '+' if w[0] >= 0 else '-'
                hh = str(int(w[1])).rjust(2, ' ')
                mm = str(int(w[2])).rjust(2, '0')
                ss = str(int(w[3])).rjust(2, '0')
                self.ziel_ra = sig + hh + "h " + mm + "' " + ss + '"'
                self.__screen.print_at(self.ziel_ra, 14, 5, colour=Screen.COLOUR_WHITE)
                # DE
                w = self.__mount.target.ziel_de.signed_dms()
                sig = '+' if w[0] >= 0 else '-'
                dd = str(int(w[1])).rjust(2, ' ')
                mm = str(int(w[2])).rjust(2, '0')
                ss = str(int(w[3])).rjust(2, '0')
                self.ziel_de = sig + dd + "° " + mm + "' " + ss + '"'
                self.__screen.print_at(self.ziel_de, 33, 5, colour=Screen.COLOUR_WHITE)

            if self.__mount.target.target_achsen_modus in (self.__mount.target.TARGETACHSENMODUS_ALTAZ,
                                                           self.__mount.target.TARGETACHSENMODUS_RADEC
                                                           ):
                # AZ
                w = self.__mount.target.ziel_az.signed_dms()
                dd = str(int(w[1])).rjust(3, ' ')
                mm = str(int(w[2])).rjust(2, '0')
                ss = str(int(w[3])).rjust(2, '0')
                self.ziel_az = dd + "° " + mm + "' " + ss + '"'
                self.__screen.print_at(self.ziel_az, 14, 6, colour=Screen.COLOUR_WHITE)
                # EL
                w = self.__mount.target.ziel_el.signed_dms()
                sig = '+' if w[0] >= 0 else '-'
                dd = str(int(w[1])).rjust(2, ' ')
                mm = str(int(w[2])).rjust(2, '0')
                ss = str(int(w[3])).rjust(2, '0')
                self.ziel_el = sig + dd + "° " + mm + "' " + ss + '"'
                self.__screen.print_at(self.ziel_el, 33, 6, colour=Screen.COLOUR_WHITE)

            # Modus ------------------------------------------------------------------
            if __oldtarget != self.__mount.target.target:
                self.__screen.print_at("                            ", 10, 8,
                                       colour=Screen.COLOUR_BLACK, bg=Screen.COLOUR_BLACK)
                __oldtarget = self.__mount.target.target

                if self.__mount.target.target is None:
                    self.__screen.print_at("automatik", 10, 8,
                                           colour=Screen.COLOUR_GREEN, bg=Screen.COLOUR_BLACK)

                elif self.__mount.target.target == self.__mount.target.TARGET_SUN:
                    self.__screen.print_at("Sonne", 10, 8,
                                           colour=Screen.COLOUR_YELLOW, bg=Screen.COLOUR_BLACK, attr=Screen.A_BOLD)

                elif self.__mount.target.target == self.__mount.target.TARGET_MOON:
                    self.__screen.print_at("Mond", 10, 8,
                                           colour=Screen.COLOUR_YELLOW, bg=Screen.COLOUR_BLACK, attr=Screen.A_BOLD)

                elif self.__mount.target.target == self.__mount.target.TARGET_JUPITER:
                    self.__screen.print_at("Jupiter", 10, 8,
                                           colour=Screen.COLOUR_YELLOW, bg=Screen.COLOUR_BLACK, attr=Screen.A_BOLD)

                elif self.__mount.target.target == self.__mount.target.TARGET_TRACKING:
                    self.__screen.print_at("tracking", 10, 8,
                                           colour=Screen.COLOUR_YELLOW, bg=Screen.COLOUR_BLACK, attr=Screen.A_BOLD)

                else:
                    self.__screen.print_at(self.__mount.target.target_name, 10, 8,
                                           colour=Screen.COLOUR_GREEN, bg=Screen.COLOUR_BLACK)

            # Nachtposition
            if self.__mount.target.target_nachtposition:
                self.__screen.print_at("Nachtpos.", 0, 6,
                                       colour=Screen.COLOUR_YELLOW, bg=Screen.COLOUR_BLACK, attr=Screen.A_BOLD)
            else:
                self.__screen.print_at("         ", 0, 6,
                                       colour=Screen.COLOUR_YELLOW, bg=Screen.COLOUR_BLACK, attr=Screen.A_BOLD)

            # Standort
            w = self.__mount.config.standort_laenge.signed_dms()
            if w != __old_laengengrad:
                __old_laengengrad = w
                ow = 'E' if w[0] > 0 else 'W'
                dd = str(int(w[1])).rjust(3, ' ')
                mm = str(int(w[2])).rjust(2, '0')
                self.laengengrad = dd + "° " + mm + "' " + ow
                self.__screen.print_at(self.laengengrad, 15, 11, colour=Screen.COLOUR_WHITE)

            w = self.__mount.config.standort_breite.signed_dms()
            if w != __old_breitengrad:
                __old_breitengrad = w
                ns = 'N' if w[0] >= 0 else 'S'
                dd = str(int(w[1])).rjust(2, ' ')
                mm = str(int(w[2])).rjust(2, '0')
                self.breitengrad = dd + "° " + mm + "' " + ns
                self.__screen.print_at(self.breitengrad, 35, 11, colour=Screen.COLOUR_WHITE)

            # Zeit
            t = time.strftime("%d.%m.%Y  %H:%M:%S", time.gmtime())
            self.__screen.print_at(t, 10, 10, colour=Screen.COLOUR_WHITE)

            # Sensoren
            az = str(self.__mount.az.sensorvalue).ljust(8, ' ')
            col = Screen.COLOUR_WHITE if self.__mount.az.sensor_issynchronized else Screen.COLOUR_BLUE
            self.__screen.print_at(az, 14, 13, colour=col)

            el = str(self.__mount.el.sensorvalue).ljust(8, ' ')
            col = Screen.COLOUR_WHITE if self.__mount.el.sensor_issynchronized else Screen.COLOUR_BLUE
            self.__screen.print_at(el, 33, 13, colour=col)

            self.__display_ratecompass()

            if self.__mount.exit_clipeum:
                for i in (1, 4, 7, 9, 14):
                    self.__screen.print_at("Q U I T   C L I P E U M - !! PLEASE WAIT !!", 1, i
                                           , colour=Screen.COLOUR_RED, attr=Screen.A_BOLD)

            self.__screen.refresh()


if __name__ == '__main__':
    try:
        s = status.Status()
        m = Terminal(s)

        s.modus = s.MODUS_AUTO
        time.sleep(2.5)
        s.modus = s.MODUS_SONNE
        time.sleep(2.5)
        s.modus = s.MODUS_MOND
        time.sleep(2.5)
        s.modus = s.MODUS_TRACK
        time.sleep(2.5)
        s.modus = s.MODUS_LOCK
        time.sleep(2.5)
        s.modus = s.MODUS_UNLOCK

        while True:
            time.sleep(0.2)
            pass
    except KeyboardInterrupt:
        pass
    except:
        pass
    finally:
        m.close()
